/** Depot4 (c) J. Lampe 1996-2010
    BrPt.java

		Branch point definition.
 		Branch points describe the parser's state in 'branch' situations, i.e., if there
 		is more than one way to proceed.
		Implements a space saving strategy as branch points are used very
		frequently but in a strict stack like manner (in fact, the embody
		the 'parsing stack').
		As object creation is a costly operation this will speed up the parser, too.

 * @author	Juergen Lampe
 * @version  1.9.4
 */

package Dp4;

class BrPt{

	/** Chain of recycled objects */
	private static BrPt freeObjectStore;

	// parse state (inlined)
	private boolean A;
	private InSt inp;
	private Source I;
	private int sk;
	private boolean ab, nb;
	private char ac, nc;

	public SourcePos pos;		// parse position in source
	protected BrPt pre;			// link to the following branch point

	/** Constructor */
	protected BrPt(){}

	/** Saves current parsing state to branch point */
	public void saveState(final Prs0 prs0){
		pos= prs0.currentSource!=null ? prs0.currentSource.GetPos() : null;
		A= prs0.A;
		ac= prs0.ac;
		nc= prs0.nc;
		ab= prs0.ab;
		nb= prs0.nb;
		inp= prs0.inp;
		I= prs0.currentSource;
		sk= prs0.sk;
	}

	/** Restores the parsing state from this branch point's data */
	public void restoreState(final Prs0 prs0){
		prs0.A = A;
		prs0.ac= ac;
		prs0.nc= nc;
		prs0.ab= ab;
		prs0.nb= nb;
		prs0.inp= inp;
		prs0.currentSource = I;
		prs0.sk= sk;
		if (prs0.currentSource!=null){
			prs0.currentSource.SetPos(pos);
		}
	}

	/** Returns a new or recycled object */
	static BrPt newBrPt(){
		BrPt result; 
		if (freeObjectStore==null){
			// no free object available
			result= new BrPt();
		} else {
			result= freeObjectStore; 
			freeObjectStore= freeObjectStore.pre; 
			result.pre= null; 
		}
		return result;
	}

	/** Clears/frees this branch point */
	static void clrBrPt(final BrPt b) {
		b.I= null;
		b.inp= null;
		b.pos= null;
		// recycle unused object
		b.pre= freeObjectStore; 
		freeObjectStore= b;
	}

	/** Clears/frees this and all following branch points */
	static void cutBrPt(final BrPt brPoint) {
		if (brPoint!=null) { 
			BrPt h= brPoint; 
			while (h.pre!=null) {
				h=h.pre;
			}
			h.pre= freeObjectStore;
			freeObjectStore= brPoint; 
		}
	}
}
