/** Depot4 (c) J. Lampe 1996-2010
    Chars.java

		Packages 'character handling related' functionality.
		It contains also the basic extension handling functionality.

 * @author	Juergen Lampe
 * @version  1.9.4
 * $Date: 2010/05/07 $
 */

package Dp4;

import java.util.HashMap;
import java.util.Map;

public class Chars{
	
	private static final int extensionPropertyKey= Translator.getPropertyKey("Dp4.ExtHandlers");
	private static LogStub log= LogStub.getLog(Chars.class.getName());

	//---------------------------------- appStr ---------------------------------
	/** Append a String to a target node */
	public static tNode appStr (tNode t, String a) {
		if (a!=null&&a.length()>0) {
			if (t!=null) {
				tNode h= tNode.thistNd(t);
				if (h instanceof XtNode) {
					String tmp= ((XtNode)h).getXdsc();
					tmp+= a;
					((XtNode)h).setXdsc(tmp);
				}else { 
					if (h instanceof CharacterTarget) {  
						((CharacterTarget)h).appendStr(a); 
					}else {
						tNode.apptN(t, new ChrtNode (a));
					}
				}
				return t;
			}
			t= tNode.apptN(null, new ChrtNode (a));
		}
		return t;
	}
	//---------------------------------- appInt ---------------------------------
	/** Append an integer value to a target node 
	 */
	public static tNode appInt(tNode t, int i) {
		return appStr(t, Integer.toString(i));
	}
	//--------------------------------- NoOfChrs --------------------------------
	/** Returns the length (number of characters) of the given target's
	 * String representation.
	 */
	public static int NoOfChrs(tNode Nd) {
		scParam p= new scParam(); 
		tNode.alltNodes(Nd, p); 
		return (int)p.length();
	}
	/** Wrapper method for source text compatibility of the meta-language */
	public static int Len(String s){ 
		return s.length();
	}
	//---------------------------------- mkChrs ---------------------------------
	/** Returns a String representation of the target */
	public static String mkChrs (tNode Nd) {
		StrBuf s= new StrBuf("mkChrs"); 
		StreamTrg.Tar2Strm(Nd, s);
		return s.toString();
	}

	/**
	 * @return a String representation of the target */
	public static String MkSym (tNode Nd) {
		StrBuf s= new StrBuf("mkChrs"); 
		StreamTrg.Tar2Strm(Nd, s);
		return s.toString();
	}
	//---------------------------------- appChn ---------------------------------
	/** Appends another target (chain) to the current target.
	 * Essentially, it is this method that makes the target structure a tree.
	 */
	public static tNode appChn(tNode t, tNode a) {
		if (t!=null) {
			tNode tt= tNode.thistNd(t);
			if (tt instanceof XtNode) { 
				XtNode h= (XtNode)tt;
				h.getXdsc().concat(mkChrs(a)); 
				return t;
			}
		}
		return tNode.appChn(t, a);
	}
	//---------------------------------- appSrc ---------------------------------
	/** Appends a source chunk identified by start and end position to the target.
	 */
	public static tNode appSrc(tNode t, SourcePos b, SourcePos e) {
		return tNode.apptN(t, new srctNode(b, e));
	}
	//-------------------------------------------------------------------------------------
	// Extension handling
	//-------------------------------------------------------------------------------------
	/** Get the current extension handlers
	 */
	public static Map getExtensionHandlers() {
		return getExtensionHandlers(Translator.getCurrentTranslator());
	}

	public static Map getExtensionHandlers(Translator translator) {
		Map map= (Map)translator.getValueFor(extensionPropertyKey);
		if (map==null) {
			map= new HashMap();
			translator.setValueFor(extensionPropertyKey, map);
		}
		return map;
	}

	//----------------------------------- appXBeg -------------------------------
	/** Appends an extension's node.
	 * Subsequent appends aim at this node until appXEnd is called.
	 */
	public static tNode appXBeg(tNode t) {
		tNode h= t;
		if (t==null) 
			h= new tNode();
		return tNode.apptN(h, new XtNode(tNode.thistNd(h)));
	}
	//------------------------------------ appXEnd ------------------------------
	/** Ends creation of an extension's node.
	 * @see #appXBeg(tNode)
	 */
	public static tNode appXEnd(tNode tn) { 
		int sp; 
		String id, para; 
		tNode t= tNode.thistNd(tn);
		XtNode h= t instanceof XtNode?(XtNode)t:null;
		h.getPre().nxt= null;
		sp= h.getXdsc().indexOf(" ", 0);
		if (sp>-1) {
			id= h.getXdsc().substring(0, sp); 
			para= h.getXdsc().substring(sp+1);
		}
		else {
			id= h.getXdsc(); 
			para= "";
		}
		Object obj= getExtensionHandlers().get(id);
		if (obj instanceof appStrT) {
			((appStrT)obj).app(h.getPre(), para);
		}
		return tn;
	}
	//------------------------------- InstallXapp -------------------------------
	/** Installs a handler for extension's nodes.
	 * This will overwrite any preceding setting for the same identifier.
	 * @param extensionIdent the extension identifier
	 * @param extensionHandler the handler
	 */
	public static void InstallXapp(String extensionIdent, appStrT extensionHandler) {
		getExtensionHandlers().put(extensionIdent, extensionHandler);
	}

	public static void InstallXapp(Translator translator, String extensionIdent, appStrT extensionHandler) {
		getExtensionHandlers(translator).put(extensionIdent, extensionHandler);
	}

	/** Conversion functions */
	//---------------------------------Int2Str ----------------------------------
	public static String Int2Str(int i){ 
		return Integer.toString(i); 
	}
	//---------------------------------Str2Int ----------------------------------
	public static int Str2Int(String s){
		try {
			return Integer.parseInt(s);
		} catch (NumberFormatException e) {
			log.log(2, "Str2Int:"+s, e);
			return Integer.MIN_VALUE;
		}
	}
	//---------------------------------- Str2tNd ---------------------------------
	public static tNode Str2tNd(String s) {
		return appStr(null, s);
	}
	//---------------------------------- Txt2Int ---------------------------------
	public static int Txt2Int(tNode t) { 
		String chrs= null;
		try {
			chrs= mkChrs(t);
			return Integer.parseInt(chrs);
		} catch (NumberFormatException e) {
			log.log(2, "Str2Int:"+chrs, e);
			return Integer.MIN_VALUE;
		}
	}
}

//-------------------------------------# scParam #-----------------------------
/** Implements a special kind of target.
 * If s is set to null it, actually, doesn't generate any text
 * but counts the characters that would be generated.
 * If s is an instance of String, the generated Strings are appended.
 */

class scParam implements Target{
	private long i= 0; 
	private String s= null;

	scParam (String S) { 
		s= S; 
	}

	scParam(){}

	public void Handler(tNode d){
		CharacterTarget Nd= d instanceof CharacterTarget?(CharacterTarget)d:null;
		if (Nd==null) 
			return;
		if (s==null) {  
			i+= Nd.getNoOfCharacters();
		}else {
			s= s+Nd.getString();
		}
	}

	public long length() {
		return i;
	}
}
