/** Depot4 (c) J. Lampe 1996-2010
Compile.java

		This class handles the compiling of java sources.
		<p>
		Compiling can be controlled by system properties.
		<dl>
		<DT>depot4.work.dir <dd>The working directory of the current application, i.e. this
		directory must be in the search path of the VM where the generated translator is to be
		executed in.
		<dt>depot4.path <dd>The path where to find Depot4 system classes. 
		The value of java.class.path will always be appended to this.
		</dl>

    * @author  Juergen Lampe
    * @version  1.9.4
    * Date: 26.2.2003
    */

package Dp4;

import java.io.File;

public class Compile {


	/** If set true, compilation will be started in an external process.
	 * Default is false.
	 */
	public static void setCompileExtern(boolean t) {
		CompilerFactory.setCompileExtern(t);
	}

	static String getWorkingDirectory() {
		String result= ".";
		try { 
			String path= System.getProperty("depot4.work.dir");
			if (path!=null)
				result= path;
		} catch (SecurityException se) {
            throw  new RuntimeException("property access", se);
        }
		return result;
	}
	
	static String getClasspath() {
		String result= ".";
		try {
			String depotPath= System.getProperty("depot4.path");
			String javaPath= System.getProperty("java.class.path");	
			if (depotPath!=null) {
				if (javaPath==null || javaPath.indexOf(depotPath)==-1)
					result= depotPath;
			}
			if (javaPath!=null)
				result+= File.pathSeparator+javaPath;
		} catch (SecurityException se) {
            throw  new RuntimeException("property access", se);
        }
		return result;
	}

	public static void Compile(String arg1){
		String args[]= {"-d", getWorkingDirectory(), "-classpath", getClasspath(), arg1};
		Compile(args);
	}
	
	public static void Compile(String arg1, String arg2){
		if (arg1==null || arg1.length()==0) {
			Compile(arg2);
		} else
		if (arg2==null || arg2.length()==0) {
			Compile(arg1);
		} else {
			String args[]= {"-d", getWorkingDirectory(), "-classpath", getClasspath(), arg1, arg2};
			Compile(args);
		}
	}
	 
	public static void Compile(final String args[]) {
		HostCompiler hostCompiler= CompilerFactory.createCompiler();
		OP.WrStr(hostCompiler.getCompilerName()+" ");
		for (int i= 0; i<args.length;i++) {
			args[i]= args[i].replace('/', File.separatorChar);
			OP.WrStr(args[i]+" ");
		}
		OP.WrLn();

		String out= hostCompiler.compile(args);

		if (out.length()==0) {
			OP.WrStr("done\n");
		} else {
			OP.WrStr(out);
		}

		System.gc();
	}

}