package Dp4;

/** The compiler factory delievers the actual host language compiler's wrapper class.
 *
 * By default, there are three different ones:
 * <ol>
 * <li>A wrapper for the compiler in package "com.sun.tools.javac", which is new since 1.4.1</li>
 * <li>A wrapper for the compiler in package "sun.tools.javac", which is deprecated since 1.4</li>
 * <li>A wrapper for the use of the javac compiler in an external process. (For this, javac must
 *   be reachable from the current working directory.)</li>
 * </OL>
 *
 * 	The factory can be customized by system properties.
 *	<dl>
 *	<DT>depot4.java.compiler</dt><dd>The full classname of an arbitrary HostCompiler. (The
 *	class must be found from the classpath.)</dd>
 *	<dt>depot4.compile.extern</dt><dd>If the value is YES, compiling is performed in the external
 *  process.</dd>
 *	</dl>
 *
 * @author Juergen Lampe
 * @version  1.9.4
 * Date: 03.03.2003
 */

public class CompilerFactory {

	private static HostCompiler currentHostCompiler;

	static boolean compilerExtern;

	static{
		String extern= System.getProperty("depot4.compile.extern");
		setCompileExtern("YES".equalsIgnoreCase(extern));
	}

	/** Gets a compiler wrapper */
	public synchronized static HostCompiler createCompiler() {
		if (currentHostCompiler==null) {
			if (!compilerExtern) {
				String compilerClassName= System.getProperty("depot4.java.compiler");
				if (compilerClassName!=null && compilerClassName.length()>0) {
					try {
						Object obj = Class.forName(compilerClassName).newInstance();
						if (obj instanceof HostCompiler){
							currentHostCompiler= (HostCompiler)obj;
						}
					} catch (Exception ignore) {
					}
				}
				if (currentHostCompiler==null) {
					try {
						Class aClass = Class.forName("com.sun.tools.javac.Main");
						if (aClass!=null){
							currentHostCompiler= new Default14Compiler();
						}
					} catch (ClassNotFoundException e) {
                        throw  new RuntimeException("no compiler", e);
					}
				}
			} else {
				currentHostCompiler= new ExternalHostCompiler();
			}
		}
		return currentHostCompiler;
	}

	/** Sets a host compiler.
	 * @param compiler the wrapper for the compiler to be used
	 */
	public static void setHostCompiler(HostCompiler compiler){
		currentHostCompiler= compiler;
	}

	/** If set true, compilation will be started in an external process.
	 * Default is false.
	 */
	public static void setCompileExtern(boolean t) {
		if (t!=compilerExtern){
			compilerExtern= t;
			currentHostCompiler= null;
		}
	}

}

