/** Depot4 (c) J. Lampe 1996-2010
ExternalHostCompiler.java

	* This class handles the compiling of java sources in an own process.
	* (May not work on certain platforms.)
	*
	* We need a different thread for waiting for the process's completion
	* because otherwise, the external process may block if its output
	* is not read from the error stream.
	*
	* @author  Juergen Lampe
	* @version  1.9.4
	* Date: 26.2.2003
	*/

package Dp4;

import java.io.InputStream;

public class ExternalHostCompiler implements HostCompiler, Runnable{

	private Process proc;
	private volatile int exitCode;

	/** Returns the name of the compiler as used in a command line interface.
	 * @return the compiler name
	 */
	public String getCompilerName(){
		return ">javac";
	}

	/** Invokes the compiler to compile a class file.
	 * @param args classpath, working directory, or other settings, and the classfile name
	 * @return the compiler output
	 */
	public String compile(final String[] args) {
		final StringBuffer sb= new StringBuffer(128);

		String[] cmdarray= new String[args.length+1];
		System.arraycopy(args, 0, cmdarray, 1, args.length);
		cmdarray[0]= "javac";
		exitCode= Integer.MAX_VALUE;
		try {
			InputStream errorStream;
			proc = Runtime.getRuntime().exec(cmdarray);
			errorStream= proc.getErrorStream();
			Thread thread= new Thread(this);
			thread.start();
			do {
				byte[] bb= new byte[128];
				int l= errorStream.read(bb);
				for (int i = 0; i<l; i++) {
					sb.append((char)bb[i]);
				}
				if (l==-1) thread.join(); //i.e., stream was closed - now we have only to wait
			} while (exitCode==Integer.MAX_VALUE);
		} catch (Exception e) {
				OP.WrStr("Exception :"+e);
		}
		if (exitCode==0)
			sb.append("done\n");
		else {
			sb.append("error(s)\n");
		}
		return sb.toString();
	}

	/** Implements the Runnable interface */
	public void run() {
		do {
			try {
				exitCode= proc.waitFor();
				proc= null;
			} catch (InterruptedException e) {
               throw  new RuntimeException("wait interrupted", e);
			}
		} while (exitCode==Integer.MAX_VALUE);
	}

}