/** Depot4 (c) J. Lampe 1996-2010
    Files.java
 		A wrapper for Java java.io.RandomAccessFile.

 * @author   J. Lampe
 * @version  1.9.4
 * $Date$
 */

package Dp4;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

public class Files implements InStream {

	private static int iostat= 0;
	static Exception ioEx= null;
	
	protected static LogStub log= LogStub.getLog(Files.class.getName());

	protected String streamName= "<undefined>";
	protected boolean eos;
	protected RandomAccessFile file= null;
	protected int fileState= 0;  // 0 = not opened; 1 = opened; 2 = does not exist

	public Files(String s)  {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "Files("+s+")");
		}
		ioEx= null;     // clear
		createFile(prepareFileName(s));
	}

	public static String prepareFileName(final String name) {
		String fileName= name;
		if ("con".equalsIgnoreCase(fileName)) {
			return fileName;
		}
		File tmpFile= new File(fileName);
		if (!tmpFile.exists()) {
			boolean isAbsolute= false;
			File[] roots= File.listRoots();
			for (int i= 0; i < roots.length; i++) {
				if (fileName.startsWith(roots[i].getPath())) {
					isAbsolute= true;
					break;
				}
			}
			if (!isAbsolute) {
				fileName= System.getProperty("user.dir") + File.separator + name;
			}
			fileName= fileName.replace(File.separatorChar, '/');
			tmpFile= new File(fileName);
			if (!tmpFile.exists()) {
				tmpFile= new File(tmpFile.getParent());
				if (!tmpFile.exists()) {
					tmpFile.mkdirs();
				}
			}
		}
		if (log.isLogEnabled(-1)) {
			log.log(-1, "prepareFileName("+name+")="+fileName);
		}
		return fileName;
	}

	public static void setIOError(int i){
		iostat= i;
	}
	public static int ioStat(){
		if (ioEx==null) {
			return 0;
		} else {
			return 1;
		}
	}

	public static String getError(){
		return ioEx.toString();
	}

	public static InStream Old(String s)  {
		return new Files(s);
	}
	public static OutStream New(String s){
		return new FileTrg(s);
	}

	private void createFile(String fileName) {
		eos= false;
		try {
			File bFile= new File(fileName);
			file= new RandomAccessFile(bFile, "r");
			fileState= 1;
			if (log.isLogEnabled(-1)) {
				log.log(-1, "created:"+fileName);
			}
			streamName= bFile.getName();
		} catch (FileNotFoundException ioex) {
			fileState= 2;
			log.log(-1, "createFile "+fileName, ioex);
			if (iostat==0) {
				OP.WrStr("'"+fileName+"'");
				OP.ERROR(34);
			} else {
				ioEx= ioex;
			}
		} catch (Exception ioex) {
			log.log(-1, "createFile "+fileName, ioex);
			if (iostat==0) {
				OP.WrStr("'"+fileName+"'");
				OP.ERROR(34);
			} else {
				ioEx= ioex;
			}
		}
	}

	public void Open(){
		if (fileState<2) {
			ioEx= null; // clear
			if (file == null) {
				createFile(prepareFileName(streamName));
			} else {
				try {
					file.seek(0);
				} catch (Exception ioex) {
					log.log(1, "Open " + streamName, ioex);
					if (iostat == 0) {
						if (ioex instanceof IOException) {
							OP.ERROR(803);
						}
					} else {
						ioEx= ioex;
					}
				}
			}
		}
	}
	public char Read() {
		ioEx= null;     // clear
		if (fileState>1) {
			return '\000';
		}
		try {
			int c= file.read();
			if (c==-1)
				return '\000';
			if (c=='\r') {
				long p= file.getFilePointer();
				c= file.read();
				if (c!='\n') file.seek(p);  // skip only if DOS line-end
				return '\n';
			}
			return (char)c; }
		catch (Exception ioex) {
			log.log(1, "Read", ioex);
			if (iostat!=0) {
				ioEx= ioex;
			}
			return '\000';
		}
	}
	public int Len() {
		if (fileState>1) {
			return 0;
		}
		try {
			ioEx= null;     // clear
			return (int)file.length();
		} catch (Exception ioex) {
			log.log(1, "Len", ioex);
			if (iostat!=0) {
				ioEx= ioex;
			}
			return 0;
		}
	}
	public int Get() {
		ioEx= null;     // clear
		if (fileState>1) {
			return 0;
		}
		try {
			return (int) file.getFilePointer();}
		catch (Exception ioex) {
			log.log(1, "Get", ioex);
			if (iostat==0) {
				if (ioex instanceof IOException)
					OP.ERROR(801);
			} else {
				ioEx= ioex;
			}
			return 0;
		}
	}
	public void Set(int p) {
		ioEx= null;     // clear
		if (fileState>1) {
			return;     // should be an error if p>0
		}
		try { 
			file.seek((long)p); 
		} catch (Exception ioex) {
			log.log(1, "Set", ioex);
			if (iostat==0) {
				if (ioex instanceof IOException){
					OP.ERROR(802);
				}
			} else {
				ioEx= ioex;
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see Dp4.InStream#Description()
	 */
//	@Override
	public String Description() {
		return streamName;
	}

	public void Close() {
		if (fileState==1) {
			fileState= 0;
		}
		if (file==null) return;
		ioEx= null;     // clear
		try {
			file.close();
			file= null;
		} catch (Exception ioex) {
			log.log(1, "Close", ioex);
			if (iostat!=0) {
				ioEx= ioex;
			}
		}
	}
	
	protected void finalize() throws Throwable {
		Close();
		super.finalize();
	}

	//------------------------- IFile -----------------------------
	public static Source IFile(String s) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "IFile("+s+")");
		}
		return StrmToken.StrmSrc(new Files(s));
	}

}
