/** Depot4 (c) J. Lampe 1996-2010
*/

package Dp4;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/** Simple thin wrapper to Commons Logging - a part of Apache Commons (TM).
 * Apache Commons and Commons Logging are trademarks of The Apache Software Foundation. 
 * Commons Logging is not part of this software. For terms of use see www.apache.org
 * 
 * Severity mapping: <-1: trace
 *                    -1: debug
 *                     0: info
 *                     1: warn
 *                     2: error
 *                    >2: fatal
 * 
 * @author lampe
 * @version
 */
public class LogStubCommonsFactory implements LogStubFactory {

	/** Initialize the logging infrastructure
	 */
	public LogStubCommonsFactory() {
		LogFactory.getLog(LogStubCommonsFactory.class);
	}

	/* (non-Javadoc)
	 * @see Dp4.LogStubFactory#getLog(java.lang.String)
	 */
	public LogStub getLog(String s) {
		Log log= LogFactory.getLog(s);
		return new CommonsLogStub(log);
	}
	
	static class CommonsLogStub extends LogStub {
		
		private final Log log;

		public CommonsLogStub(Log log) {
			this.log= log;
		}

		public boolean isLogEnabled(int severity) {
			switch (Math.max(severity, -2)) {
    			case -2: return log.isTraceEnabled();
    			case -1: return log.isDebugEnabled();
    			case  0: return log.isInfoEnabled();
    			case  1: return log.isWarnEnabled();
    			case  2: return log.isErrorEnabled();
    			default: return log.isFatalEnabled();
			}
		}

		public void log(int severity, String msg, Throwable ex) {
			switch (Math.max(severity, -2)) {
    			case -2: log.trace(msg, ex); break;
    			case -1: log.debug(msg, ex); break;
    			case  0: log.info(msg, ex); break;
    			case  1: log.warn(msg, ex); break;
    			case  2: log.error(msg, ex); break;
    			default: log.fatal(msg, ex);
    		}
		}

		public void log(int severity, String msg) {
			switch (Math.max(severity, -2)) {
    			case -2: log.trace(msg); break;
    			case -1: log.debug(msg); break;
    			case  0: log.info(msg); break;
    			case  1: log.warn(msg); break;
    			case  2: log.error(msg); break;
    			default: log.fatal(msg);
    		}
		}
		
	}

}
