/** Depot4 (c) J. Lampe 1996-2010

Mapper.java

		Manages mappings between String objects.

 * @author	Juergen Lampe
 * @version  1.9.4
 * $Date: 2010/05/19 $
 */

package Dp4;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Mapper {

	private static final int propertyKey= Translator.getPropertyKey("Dp4.Mapper");

	protected HashMap mapper= new HashMap();
	protected List mapperStack= new ArrayList(2);

	/** Constructor */
	public Mapper(){}

	protected static Mapper getCurrentMapper(Translator translator) {
		Mapper map= (Mapper)translator.getValueFor(propertyKey);
		if (map==null) {
			map= new Mapper();
			translator.setValueFor(propertyKey, map);
		}
		return map;
	}

	//---------------------------- getMap ----------------------------------
	/** Returns the mapped String as a target object within the current Mapper.
	 */
	public static tNode getMap(String s){
		return Chars.appStr(null, 
				getCurrentMapper(Translator.getCurrentTranslator()).get(s));
	}

	//---------------------------- getMap ----------------------------------
	/** Returns the mapped String as a target object within the translator's current Mapper.
	 */
	public static tNode getMap(Translator translator, String s){
		return Chars.appStr(null, getCurrentMapper(translator).get(s));
	}

	//---------------------------- getSMap ------------------------------- 
	/** Returns the mapped String within the current Mapper.
	 */
	public static String getSMap(String s){
		return getCurrentMapper(Translator.getCurrentTranslator()).get(s);
	}

	//----------------------------- setMap ----------------------------------
	/** Define a new mapping within the current Mapper.
	 */
	public static void setMap(String s, String t){
		getCurrentMapper(Translator.getCurrentTranslator()).put(s, t);
	}

	//----------------------------- setMap ----------------------------------
	/** Define a new mapping within the current Mapper.
	 */
	public static void setMap(Translator translator, String s, String t){
		getCurrentMapper(translator).put(s, t);
	}

	public String get(String s) {
		String result= (String)mapper.get(s);
		if (result==null)
			result= s;
		return result; 
	}

	public void put(String s, String t){
		mapper.put(s, t);
	}

	//---------------------------------------------------------	
	public void rstMaps(int m){ // restore mappings
		mapper= (HashMap)mapperStack.get(m);
	}

	public int svMaps(){  // save mappings and open new
		mapperStack.add(mapper);
		mapper= new HashMap();
		return mapperStack.size()-1;
	}
	public void clrMaps(){ //clear mappings
		mapper.clear();
	}
} 
