/** Depot4 (c) J. Lampe 1996-2003
    Messages.java
    
    * @author  Juergen Lampe
    * @version 1.9.2
    * date:    06.03.2003
    */
    
package Dp4;

import java.util.Vector;
import java.util.HashMap;

public class Messages{

	/** System messages */
  public static final String [] systemMessages= {
	/* 0*/  "*Loading ",
	/* 1*/  " failed",
	/* 2*/  " initiated",
	/* 3*/  "  unloaded",
	/* 4*/  "-call   ",
	/* 5*/  "-return ",
	/* 6*/  "** errors - compilation denied",
	/* 7*/  "-- Logging ",
	/* 8*/	"Lexer settings ",	
	/* 9*/	"Distinguish uppper/lower case =",	
	/*10*/	"Comment def: ",	
	/*11*/	"Module instantiation: ",	
	/*12*/	"Module loading: ",	
	/*13*/	"Load exceptions\n  ",	
	/*14*/	"Unload notification exception: ",	
	/*15*/	"There is no configuration file: ",	
	/*16*/	"Unknown code=",
	/*17*/	"Could not set messages: ",
	/*18*/	"Reading configuration file: ",
  };
	/** System error messages */
  private static String [] err= {
	/* 0*/  "",
	/* 1*/  " Unexpected end of source",
	/* 2*/  " Missing branch point",
	/* 3*/  " Type mismatch",
	/* 4*/  " Invalid parameter(s)",
	/* 5*/  " Syntax error",
	/* 6*/  " Unable to open file",
	/* 7*/  " End-of-text exceeded",
	/* 8*/  " Comment not closed",
	/* 9*/  " Internal list overflow",
	/*10*/  " String too long",
	/*11*/  " Too many globals used",
	/*12*/  " Global not declared",
	/*13*/  " Missing implementation in module",
	/*14*/  " Case control out of range",
	/*15*/  ".. has different type",
	/*16*/  " ", //": FLEX index <=0 (changed to 1) ", // no longer used
	/*17*/  " Undefined target used",
	/*18*/  " Undeclared identifier",
	/*19*/  " Multiply defined identifier",
	/*20*/  " Identifier does not match module name",
	/*21*/  " \";\" missing",
	/*22*/  " Default \"N\" used more than once",
	/*23*/  " Default \"O\" used more than once",
	/*24*/  " Default \"c\" used more than once",
	/*25*/  " Missing target specification",
	/*26*/  " Number too large",
	/*27*/  " Too many comment types defined",
	/*28*/  " No comment type left to remove",
	/*29*/  " Illegal character in number",
	/*30*/  " SOURCETEXT across From or Back",
	/*31*/  " ", //" \"From\" not supported by Dp4Light", // no longer used
	/*32*/  " Illegal character in string",
	/*33*/  " not available - implementation restriction",
	/*34*/  " No such file"
  };

	private static Messages defaultInstance= new Messages();

	protected Vector errorMessages = new Vector();
	protected HashMap errorTextMap = new HashMap(100);

	protected Messages(){
		init();
	}

	/** Overwrite in subclasses to set modfied (translated) system messages,
	 *  and/or to define custom messages.
	 *  This method adds the system error messages.
	 */
	protected void init(){
		for (int i = 0; i<err.length; i++) {
			errorMessages.add(err[i]);            // do it the old way
			errorTextMap.put("dp4."+i, err[i]);   // do it the new way
		}
	}

	public static synchronized Messages getMessages(){
		if (defaultInstance==null){
			defaultInstance= new Messages();
			defaultInstance.init();
		}
		return defaultInstance;
	}

	/** Use this method to set an own message handler */
	public static void setMessages(Messages m){
		defaultInstance= m;
	}

	/** Use this method to set an own message handler */
	public static void setMessages(String className) throws Exception {
			Object messages = Class.forName(className).newInstance();
			setMessages((Messages)messages);
	}

	/** Returns the error text associated to a number.
	 * This should not be used any longer.
	 */
	public static String getErrorTxt(int idx){
		try {
			return (String)getMessages().errorMessages.get(idx);
		} catch (Exception e) {
			return " "+systemMessages[16]+idx;
		}
	}

	/** Returns the error message for an error key.
	 * @see #setErrorTxt
	 * @param key the error key
	 * @return the message text
	 */
	public static String getErrorTxt(String key){
		try {
			Object txt = getMessages().errorTextMap.get(key);
			if (txt instanceof String){
				return (String)txt;
			}
		} catch (Exception e) {}
		return " "+systemMessages[16]+key;
	}

	/** Defines the error message for an error key.
  	* @param key a short key for the message; to avoid unintended interaction, keys
	 	* 					should start with an indicator such as "dp4.", which is used for system errors
 		* @param txt the associated message text
 		*/
	public static void setErrorTxt(String key, String txt){
		getMessages().errorTextMap.put(key, txt);
	}
}
