/** Depot4 (c) J. Lampe 1996-2010
    ModLoader.java
 *
 * This is the central class for all class loading operations.
 * It handles the various path and package settings for translator classes.
 *
 * @author	Juergen Lampe
 * @version  1.9.4
 * $Date: 2010/06/02 $
 */

package Dp4;

import java.util.ArrayList;
import java.util.List;

public class ModLoader {
	private static List modNames= new ArrayList(20), 
	                    modInst = new ArrayList(20),
		                modFree = new ArrayList(4);
	// Assert: numOfPaths+1 >= paths.length !!
	private static String[] paths= {"", null, "Dp4", null, null, null, null};
	private static int initialNumOfPaths= 2;
	private static int numOfPaths= initialNumOfPaths;
	private static List packageNames= new ArrayList(4);
	protected static int LdErr= 0;
	private static final String logHead= Messages.systemMessages[7]+Messages.systemMessages[13];

	static{
		String workingDir= getWorkingDirectory();
		if (workingDir!=null && workingDir.length()>0){
			setModPath("file://"+workingDir);
			initialNumOfPaths= numOfPaths;
		}
		setPackages(getPackageNames());
	}

	public static String getWorkingDirectory() {
		String result= "";
		try { 
			result= System.getProperty("depot4.work.dir");
		} catch (SecurityException se) {}
		return result;
	}

	public static String[] getPackageNames() {
		String[] result= new String[0];
		try {
			String all= System.getProperty("depot4.trans.packages");
			if (all!=null && all.length()>0){
				List tmp= new ArrayList();
				int i1= 0;
				int i2= all.indexOf(';');
				if (i2==-1) i2= all.length();
				while (i2!=-1 && i2-i1>1) {
					tmp.add(all.substring(i1, i2));
					i1= i2+1;
					i2= all.indexOf(';', i1);
					if (i2==-1 && all.length()>i1) i2= all.length();
				}
				result= new String[tmp.size()];
				result= (String[])tmp.toArray(result);
			}
		} catch (SecurityException se) {}
		return result;
	}

	//----------------------------------- setModPath ----------------------------------
	/**
	 * @param path the search path (use / as separator character!)
	 */
	public static void setModPath(String path) {
		if (numOfPaths+1>=paths.length){
			String [] pa= new String[numOfPaths+4];
			System.arraycopy(paths, 0, pa, 0, paths.length);
			paths= pa;
		}
		if (path==null||path.length()==0)
			numOfPaths=initialNumOfPaths;
		else 
			paths[numOfPaths++]= path;
		// this is always the last try:
		paths[numOfPaths]= "Dp4";
	}

	/** Sets package names where to search for translator classes.
	 * @param packs array of package names; null clears all settings
	 */
	public static void setPackages(final String[] packs){
		packageNames.clear();
		if (packs!=null) {
			for (int i = 0; i<packs.length; i++) {
				packageNames.add(packs[i]);
			}
		}
	}

	/** Adds a package name to the list */
	public static void addPackage(String pack){
		if (pack!=null && pack.length()>0 && !packageNames.contains(pack)){
			packageNames.add(pack);
		}
	}

	public static void loadModule(String n, Translator translator) {
		NTprocedure ntm= ldModule(n);
		if (ntm!=null)
			if (ntm instanceof NTprocedure)
				ntm.init(translator);
	}

	static synchronized NTprocedure ldModule(String modName) {
		Class ntc= null; 
		NTprocedure ntm= null;
		int fr= -1;
		final int moduleIdx= modNames.indexOf(modName); // moduleIdx must stay unchanged !!

		if (moduleIdx!=-1){
			//i.e. module already there - now check if invalidated
			fr= modFree.indexOf(modName);
		}
		if (moduleIdx!=-1&fr==-1) {
			// there and not invalidated, moduleIdx.e. ready
			ntm= (NTprocedure)modInst.get(moduleIdx);
		} else {
			// search for module
			// start with search for a prefix String
			int j= 0;
			String packName= Util.getPrefix(modName);
			// if there is a prefix add it to the search paths
			// position 1 is reserved for this
			if (packName.length()>0) {
				paths[1]= packName;
			} else { 
				paths[1]= "";
				j= 1;	// position 0 is always empty, so we can skip it
			}
			String tmpName= modName;
			String netName;
			int j1= j;
			load:for (int p= -1; p<packageNames.size(); p++){
				if (p>=0){
					tmpName= ((String)packageNames.get(p))+"."+modName;
				}
				for (j= j1; j<=numOfPaths; j++){
					netName= ((paths[j].length()==0)?tmpName:paths[j]+"."+tmpName);
					LdErr= 0;
					ntc= doLoad(netName, fr);
					if (Base.LogLevel.get(12))
						OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[12]+netName+((LdErr==0)?" loaded":" not found")+"\n");
					if (LdErr==0) {
						break load;
					}
				}
			}
			if (LdErr!=0) {
				OP.WrStr(Messages.systemMessages[0]+modName+Messages.systemMessages[1]+"\n");
				return null;
			}
			try { 
				ntm= ( NTprocedure)ntc.newInstance();
			} catch  (Exception notF) { 
				OP.WrStr("**ModLoader: "+notF+"\n"); 
				return null; 
			}
			if (fr==-1) { 
				modNames.add(modName);
				modInst.add(ntm);
			}else { 
				modFree.remove(fr);
				modInst.set(moduleIdx, ntm);
			}
			if (Base.LogLevel.get(2)) 
				OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[11]+modName+Messages.systemMessages[2]+"\n");
		}
		return ntm;
	}

	static synchronized Class doLoad(String n2, int fr){
		LdErr= 0;
		if (fr!=-1) {
			try { 
				return ClassReloader.forName(n2);
			} catch (Exception notF) { 
				LdErr= 1;
				if (Base.LogLevel.get(11)) OP.WrStr(logHead+notF+"\n");
			}
		} else {
			n2= n2.replace('/', '.');
			try {
				return Class.forName(n2);
			} 
			catch (Exception notF) { 
				LdErr= 1;
				if (Base.LogLevel.get(11)) OP.WrStr(logHead+notF+"\n");
			}
		} 
		return null;
	}
	//------------------------------------- FreeNT -------------------------------------
	public static synchronized void FreeNT(String a) {
		freeNT(a);
	}
	public static synchronized void freeNT(String a) {
		if (a!=null && a.length()>0) {
			int i= modNames.indexOf(a);
			if (i!=-1) modFree.add(a);
		}
	}

}
