/** Depot4 (c) J. Lampe 1996-2003
    NonTerm.java
		
		This class manages the nonterminal related data, such as the name of
		the class which actual implements its functionality, monitor state etc.
    
	* @author	Juergen Lampe
	* @version 1.9.2
	* Date:		03.06.2003
  */
    
package Dp4;

import java.util.Stack;

public class NonTerm extends NonTermT {

  public static final NTprocedure init = new initNT();
  
  private static int NTcounter= 0;

	// an unique id for every NonTerm object
  public final int id= NTcounter++;
	// the fields for the actual parser objects
	// if mon==false (default case) P is the parser class
	// otherwise, the former value of P is moved into monP and P contains the
	// monitoring parser (which in turn should invoke monP)
  public NTprocedure P, monP;
	// the monitoring flag
  public boolean mon;

	private Stack stack= new Stack();

	/** Constructs a NonTerm object
	 * @param translator the associated translator
	 * @param name the nonterminal's name (which is also the default implementation class name)
	 */
	public NonTerm(Translator translator, String name) {
    this.name= name; 
		impname= name; 
		eqname= name;
		mon= !NTprocMonT.isNotMonitoring(translator);
    if (mon) {
			monP= init;
			P= NTprocMonT.getMonAllProc(translator);
    } else 
			P= init; 
	}

	/**
	 * @return the current implementation's class name or "<init>" if non loaded yet
	 */
	public String getCurrentName() {
		NTprocedure proc;
		if (mon)
			proc= monP;
		else
			proc= P;
		if (proc==init)
			return "<init>";
		else
			return proc.getClass().getName();
	}

  /** Sets an implementation for this NT.
	 * @param p an implementation
	 */
	public void installNT(NTprocedure p) {
    if (mon)
			monP= p;
    else 
			P= p; 
  }

  /** Set this back to the inital state.
	 * @param translator the current translator
	 */
	public void clearNT(Translator translator) {
		P= init; 
		mon= false;
  }
	
  public void pushNT(String a){
  	String[] data=  {new String(impname), new String(eqname)};
  	stack.push(data);
    int s= a.indexOf(':');
    if (s==-1) {
      impname= a;
  		eqname= name;
    } else {
      impname= a.substring(s+1);
      eqname= a.substring(0, s);
  	}
    if (mon)
			monP= init;
		else
			P= init;
  }
	
  public void popNT() {
    if (stack.empty())
    	impname= name;
    else{
			String[] data= (String[])stack.pop();
    	impname= data[0];
    	eqname= data[1];
    }
    if (mon)
			monP= init;
		else
			P= init;
  }
	
  public void modNT(String a) {
		String newImpname= a, newEqname= name;
    final int s= a.indexOf(':');
    if (s!=-1) {
      newImpname= a.substring(s+1);
      newEqname= a.substring(0, s);
  	}
    if (!newImpname.equals(impname)||!(newEqname.equals(eqname))) {
  		pushNT(a);
    }
  }
	
  public void resetNT(String a) {
    if (a.equals(impname)) {
      if (mon) monP= init; 
  		else P= init;
    } 
  }
 
  static {
		NTcall.install(new NTcallDefault());
	}
}


//----------------------------------------------------------
// the initial parser implementation for every nonterminal
// causes the loading of the currently actual implementor class
class initNT extends NTprocedure {

	public trgts parse(NonTermT t, Prs p) { 
  	NonTerm nt= (NonTerm)t; 
		int LdErr;
    LdErr= p.getTranslator().loadModule(nt.impname);
    if ((nt.mon&&nt.monP==this)||(!nt.mon&&nt.P==this)) {
      if (LdErr==0) { 
				OP.WrStr(nt.impname+"."+nt.name+"  "); 
				OP.ERROR(13);
			}
      p.A= false; 
			return null;
		} else if (nt.mon)  
			return nt.monP.parse(nt, p);
    else {
			return nt.P.parse(nt, p);
    }
  }
	
}

//----------------------------------------------------------
// Default implementation of the abstract NTcall class
//
final class NTcallDefault extends NTcall{

	public NTcallDefault(){}
	
  protected void callNTbyName (Prs P, String name){
    NonTerm r; 
    r= P.getTranslator().registNT(name);
		r.P.parse(r, P);
  }

}
