/** Depot4 (c) J. Lampe 1996-2010

 The OutputProtocol processor base class.
 Applications may subclass this and set their own processors.
 This default implementation directs all output to System.out.

* @author  Juergen Lampe
* @version  1.9.4
* $Date: 2010/05/11 $
*/

package Dp4;

import java.io.PrintStream;

public class OP {

	/** This is public for compatibility to former versions only.
	 * (Should be private) */
	public static OP act= new OP();

	private PrintStream outStream;

	public OP(PrintStream out) {
		outStream = out;
	}

	public OP() {
		this(System.out);
	}

	public void setCurrentPrintStream(PrintStream out){
		outStream = out;
	}

	public static void setCurrentOP(OP op) {
		act= op;
	}

	/** Sets a Messages class by class name.
	 * @param className the name of a class which is derived from Dp4.Messages
	 */
	public static void setMessages(String className){
		try {
			Messages.setMessages(className);
		} catch (Exception e) {
			WrStr(Messages.systemMessages[17]+className+"\n      "+e);
		}
	}

	public static void WrStr(String arg) {
		act.wrStr(arg);
	}

	public static void WrInt(int n) {
		act.wrInt(n);
	}

	public static void WrLn() {
		act.wrLn();
	}

	public static void WrBt(byte[] argv) {
		act.wrBt(argv);
	}

	public static void WrChr(char c) {
		act.wrChr(c);
	}

	public static void ERROR(int i, String msg) {
		if (msg!=null && msg.length()>0){
			WrStr(msg+" ");
		}
		if (i>1000) {
			WrStr(" FATAL ERROR ");
			i-=1000;
		} else
			WrStr(" error");
		WrStr(": "+Messages.getErrorTxt(i)+"\n");
	}

	public static void ERROR(int i) {
		ERROR(i, null);
	}

	public static void ERROR(String key, String msg) {
		if (msg!=null && msg.length()>0){
			WrStr(msg+" ");
		}
		WrStr(" error: "+Messages.getErrorTxt(key)+"\n");
	}

	public static void FATALERROR(String key, String msg) {
		if (msg!=null && msg.length()>0){
			WrStr(msg+" ");
		}
		WrStr(" FATAL ERROR "+Messages.getErrorTxt(key)+"\n");
	}

	public static void ERROR(String key) {
		ERROR(key, null);
	}

	public static void WARNING(int i, String msg) {
		if (Base.LogLevel.get(6)) {
			if (msg!=null && msg.length()>0){
				WrStr(msg+" ");
			}
			WrStr(" warning:"+Messages.getErrorTxt(i)+" \n");
		}
	}

	public static void WARNING(int i) {
		WARNING(i, null);
	}

	public static void WARNING(String key, String msg) {
		if (Base.LogLevel.get(6)) {
			if (msg!=null && msg.length()>0){
				WrStr(msg+" ");
			}
			WrStr(" warning:"+Messages.getErrorTxt(key)+" \n");
		}
	}

	public static void WARNING(String key) {
		WARNING(key, null);
	}

	public static boolean WARNON() {
		return Base.LogLevel.get(6);
	}

	/** The following methods should be overwritten in subclasses:*/

	public void wrStr(String arg) {
		outStream.print(arg);
	}

	public void wrInt(int n) {
		outStream.print(n);
	}

	public void wrLn() {
		outStream.println();
	}

	public void wrBt(byte[] argv) {
		outStream.print(argv);
	}

	public void wrChr(char c) {
		outStream.print(c);
	}

}
