/** Depot4 (c) J. Lampe 1996-2010

    Parser object

    @author	Juergen Lampe
    @version  1.9.4
    $Date: 2010/05/28 $
    */

package Dp4;


public class Prs extends Prs0 {

	protected int coN= -1;
	protected CommentDesc [] commentDescs= new CommentDesc[Base.MaxComments];
	protected final Translator translator;

	protected BrPt lastBranchPoint;
	private BrPtObs branchPointObserver= null;

	public Prs(Translator translator) {
		this.translator= translator;
	}
  
	public Translator getTranslator() {
		return translator;
	}

	// ------- no more doubt ----------------------------------
	public void ND(){ 
		BrPt.cutBrPt(lastBranchPoint);
		lastBranchPoint= null;
		if (branchPointObserver!=null){
			branchPointObserver.ND(this);
		}
	}
	// ------- back track -------------------------------------
	public void BckTr(){
		if (lastBranchPoint!=null) {
			lastBranchPoint.restoreState(this);
			if (branchPointObserver!=null)
				branchPointObserver.BT(this);
		}
	}
	// ------- DelB -------------------------------------------
	public void DelB(){
		if (lastBranchPoint!=null) {
			BrPt h= lastBranchPoint;
			lastBranchPoint= lastBranchPoint.pre;
			BrPt.clrBrPt(h);
		}
		if (branchPointObserver!=null) {
			branchPointObserver.DB(this);
		}
	}
	// ------- SetB -------------------------------------------
	public void SetB(){
		final BrPt b= BrPt.newBrPt();
		b.pre= lastBranchPoint;
		lastBranchPoint= b;
		b.saveState(this);
		if (branchPointObserver!=null) {
			branchPointObserver.SB(this);
		}
	}
	// ------- Skip -------------------------------------------
	public void Skip(){
		if (sk==0) {
			while (skipSpaces() && skipComments());
		}
	}
	// ------- NoSkip -------------------------------------------
	public void NoSkip(){
		sk+= 1;
	}
	// ------- ReSkip -------------------------------------------
	public void ReSkip(){
		if (sk>0) sk-= 1;
	}
	// ------- accept char ------------------------------------
	public void acc(){ 
		ab= true;
	}
	// ------- From -------------------------------------------
	public void From(Source source){
		InSt s= new InSt();
		s.pre= inp;
		s.ts= currentSource;
		s.ab= ab; 
		s.nb= nb; 
		s.ac= ac; 
		s.nc= nc;
		ab= true;
		nb= true;
		inp= s;
		currentSource= source;
		source.Reset();
	}
	// ------- Cut --------------------------------------------
	public void Cut(){
		if (inp!=null&&inp.ts!=null) {
			currentSource= inp.ts;
			ab= inp.ab; 
			nb= inp.nb; 
			ac= inp.ac; 
			nc= inp.nc;
			inp= inp.pre;
		} else 
			nb=false;
	}
	// ------- next char --------------------------------------
	public char NC(){
		/*    while (nb) { nc= I.NxtChr();
      if (nc=='\000') Cut();
      else nb= false;        
    };*/
		nc= currentSource.NxtChr();
		if (nc=='\r') return Base.EOL;
		return nc;
	}
	// ------- current char -----------------------------------
	public char AC(){
		if (ab) { 
			ac= NC(); 
			ab= false; 
			nb= true;
		}
		return ac;
	}
	// ------- compare string ---------------------------------
	public void CS(String s){ 
		int l, k= 0, i; 
		char c;
		Skip();
		l= s.length()-1;
		if (l>0 && s.charAt(0)==Base.abbrChr && Character.isDigit(s.charAt(1))) {
			k= 2;
			i= Integer.parseInt(s.substring(1,2))+2;
			for (;;) {
				if (k>l) { 
					c= AC();
					A= !Character.isLetterOrDigit(c); 
					break;
				}
				if (s.charAt(k)==AC()) { 
					k+= 1; 
					acc();
				} else if (k<i) { 
					A= false; 
					break;
				} else { 
					c= AC(); 
					A= !Character.isLetterOrDigit(c); 
					break;
				}
			}
		}
		else {
			for (;;) {
				if (k>l) break;
				if (s.charAt(k)==AC()) { 
					k+= 1; 
					acc();
				} else { 
					A= false; 
					break;
				}
			}
		}
	}
// ------- DefCo ------------------------------------------
  public void DefCo(String ca, String ce){
	  int i, j;  
	  char sw= ' ', r= '*';
	  String sy= null;
	  if (ca.charAt(0)=='$') {
		  i= ca.indexOf("$", 1);
		  if (i!=-1) {
			  j= ca.indexOf(":", 0);
			  if (j!=-1&&j<i) { sy= ca.substring(j+1, i);
			  }
			  sw= ca.charAt(1); if (ca.charAt(2)=='-') r= '-';
			  ca= ca.substring(i+1);
		  }
		  if (sw=='-') { 
			  i= coN; 
			  coN= (coN>-1)?coN-1:-1;
			  if (Base.LogLevel.get(1)) 
				  OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[10]+"["+i+"]: removed\n");
			  return;
		  }
		  if (sw=='+') coN+= 1;
		  i= coN;
		  if (Character.isDigit(sw)) {
			  i= Character.digit(sw, 10)-1;
			  if (i==coN+1) coN+= 1;
		  }
		  if (i>=commentDescs.length-1){
			  OP.ERROR(627); 
			  return;
		  }
	  } else { 
		  coN= 0; 
		  i= 0; 
	  }
	  if (Base.LogLevel.get(1))
		  OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[10]+"["+i+"]:"+ca+"..."+ce+"'"+((sy==null)?"":sy)+"'"+r+"\n");
	  commentDescs[i]= new CommentDesc(ca, ce, sy, r);
  }

  boolean skipSpaces(){
	  char c= AC();
	  while (c<=' '&&c!='\000') {
		  acc();
		  c= AC();
	  }
	  return true;
  }

  boolean skipComments() {
	  for (int i = coN; i>-1; i--) {
		  if (CommentDesc.isComment(this, commentDescs[i])) return true;
	  }
	  return false;
  }


	//--------------------------------------------------------------------
	// static utility methods
	//--------------------------------------------------------------------

  public static void addBrPtObs(Prs P, BrPtObs B){
    B.nxt= P.branchPointObserver;
		P.branchPointObserver= B;
  }
}

/** Descriptor of a comment format */
class CommentDesc {
  private final char[] com;
  private String prag= null;
  private int coc, coe;
  private boolean rec= true;
  private char c1;
  
	//--------------------------------- is ----------------------------------
	static boolean isComment(Prs P, CommentDesc CO){
	  int s= 0;// int z= 0;
	  char C;
	  boolean db;
	  C= P.AC();
	  do {
			db= false; //z+= 1;
	    if (s==0) {
	      if (CO.com[0]==C) {
					s+= 1;
					if (s==CO.coc) {
						s+= 1;
						if (CO.prag!=null) {
							P.acc();
							NTcall.callNT(P, CO.prag);
							db= true;
						}
					} else
						P.SetB();
				} else
					return false;
	    } else {
	      if (s>CO.coc) {
	        if (CO.com[s]==C) {
	          s+= 1;
	          if (s==CO.coe) {
	            if (CO.com[CO.coe]!=C) P.acc();
	            return true;
						}
	        } else {
	          if (CO.rec && s==CO.coc+1 && CO.c1==C) {
							db= isComment(P, CO);   // recursive comments
	          } else {
							if (s>CO.coc+1) {
								s= CO.coc+1;
								db= true;
							}
						}
	        }
	      } else {
	        if (s<CO.coc) {
	          if (CO.com[s]!=C) {
							P.BckTr();
							P.DelB();
							return false;
	          } else {
							s+= 1;
	            if (s==CO.coc) {
								s+= 1;
								P.DelB();
	              if (CO.prag!=null) {
									P.acc();
									NTcall.callNT(P, CO.prag);
									db= true;
	              }
	            }
	          }
	        }
	      }
	    }
	    if (!db) P.acc();
	    C= P.AC(); //if (z>200) {OP.ERROR(888); return false;}
	    if (C=='\000') {
				OP.ERROR(8);
				return true;
			}
	  } while (true);
	}

	CommentDesc (String ca, String ce, String sy, char r){
	  prag= sy;
	  com= (ca+"#"+ce).toCharArray();
	  coc= ca.length();
	  coe= com.length-1;
	  c1= com[0];
	  rec= r=='*';
	}  
}

