/** Depot4 (c) J. Lampe 1996-2010
    Tools.java
    
    * @author  Juergen Lampe
    * @version  1.9.4
    * Date:   19.03.2003
    */

package Dp4;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;

public class Tools{

	private static ToolsFactory factory= new ToolsFactory() {
		public Tools createToolsInstance(){
			return new Tools();
		}
	};
	
	private static Tools toolsInstance;
	
	protected static LogStub log= LogStub.getLog(Tools.class.getName());
	
	/** Set a customized ToolsFactory
	 * @param f 
	 */
	public static void setToolsFactory(ToolsFactory f) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "setToolsFactory("+f.getClass().getName()+")");
		}
		factory= f;
		toolsInstance= null;
	}
	
	protected static Tools getToolsInstance() {
		if (toolsInstance==null) {
			toolsInstance= factory.createToolsInstance(); 
		}
		return toolsInstance;
	}
	
	/** Logs an exception due to the LogLevel set in Base.
	 * @param e the exception to be logged
	 */
	public static void logException(Throwable e) {
		log.log(-2, "", e);
		getToolsInstance().printException(e, e.toString());
	}
	
	/** Logs an exception due to the LogLevel set in Base.
	 * @param e the exception to be logged
	 * @param msg an additional message
	 */
	public static void logException(Throwable e, String msg) {
		log.log(-2, msg, e);
		getToolsInstance().printException(e, msg);
	}
	
	/** Prints an exception due to the LogLevel set in Base.
	 * @param e the exception to be logged
	 * @param msg an additional message
	 */
	protected void printException(Throwable e, String msg) {
		OP.WrStr(" *** "+msg+"\n"); 
		log.log(-2, "printException:"+msg, e);
		if (Base.LogLevel.get(13)) {
			OutputStream b= new ByteArrayOutputStream(512);
			e.printStackTrace(new PrintStream(b));
			OP.WrStr(b.toString());
		}
	}
	
	/** Implements the application of a translation.
	* @param translator  the associated runtime data object
	* @param s  the source text of the translation
	* @param ROOT  the root nonterminal
	* @return the generated targets' object
	* side effect: the success or failure of the translation can be checked by a call
	* to <code>translator.isOk()</code>.*/
	protected trgts execTranslate(Translator translator, Source s, NonTerm ROOT){
		if (log.isLogEnabled(-1)) {
			log.log(-1, "execTranslate("+translator+","+s+","+ROOT.getCurrentName()+")");
		}
		translator.setOk(true);
		try {
			translator.startTranslation();
			Prs P= translator.isCap()?new Prs(translator):new PrsC(translator);
			if (Base.LogLevel.get(10)) { 
				OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[9]+translator.isCap()+"\n");
			}
			P.From(s);
			P.DefCo("/*","*/ ");
			trgts tt= ROOT.P.parse(ROOT, P);
			if (!P.A) { 
				s.LogPos(); 
				OP.ERROR(5);
			}
			translator.setOk(P.A);
			return tt;
		} catch (Throwable e) {
			OP.WrStr(" *** "+e+"\n"); 
			if (Base.LogLevel.get(13)) {
				OutputStream b= new ByteArrayOutputStream(1000);
				e.printStackTrace(new PrintStream(b));
				OP.WrStr(b.toString());
			}
			log.log(1, "execTranslate", e);
			return new trgts(); 
		} finally {
			if (log.isLogEnabled(-1)) {
				log.log(-1, "finished execTranslate ok="+translator.isOk);
			}
			translator.endTranslation();
		}
	}
//------------------------------ translate ----------------------------------
  public static trgts translate(Translator translator, Source s, NonTerm ROOT){
		return getToolsInstance().execTranslate(translator, s, ROOT);
  }
//------------------------------ translate ----------------------------------
  public static trgts translate(Source s, NonTerm ROOT){
		return translate(Translator.getCurrentTranslator(), s, ROOT);
  }
//------------------------------ translate ----------------------------------
  public static trgts translate(Translator translator, Source s, String ROOT){
    return translate (translator, s, translator.registNT(ROOT));
  }
//------------------------------ translate ----------------------------------
  public static trgts translate(Source s, String ROOT){
    return translate (Translator.getCurrentTranslator(), s, ROOT);
  }
	
	public static boolean isOk() {
		return Translator.getCurrentTranslator().isOk();
	}
	
	public static void setCap(boolean is) {
		Translator.getCurrentTranslator().setCap(is);
	}
	
	public static boolean isCap() {
		return Translator.getCurrentTranslator().isCap();
	}
	
	/** The following methods do a deep copy of the values of their arguments.
	* s means always 'source', t stands for 'target'
	* Important: The target must have been created beforehand!
	*/
//--------------------------------- deepCp ----------------------------------
  public static void deepCp(int[] s, int[] t){
     System.arraycopy(s, 0, t, 0, s.length);
  } 
  public static void deepCp(boolean[] s, boolean[] t){
      System.arraycopy(s, 0, t, 0, s.length);
  } 
  public static void deepCp(float[] s, float[] t){
      System.arraycopy(s, 0, t, 0, s.length);
  } 
  public static void deepCp(Object[] s, Object[] t){
    int l= s.length;
    for (int i= 0; i<l;i++) {
      if (s[i] instanceof Object[]) deepCp((Object[])s[i], (Object[])t[i]);
      else {
        if (s[i] instanceof int[]) {
			int[] s1= (int[])s[i];
			System.arraycopy(s1, 0, ((int[])t[i]), 0, s1.length);
		} else {
          if (s[i] instanceof boolean[]) {
			boolean[] s1= (boolean[])s[i];
			System.arraycopy(s1, 0, ((boolean[])t[i]), 0, s1.length);
		} else {
            if (s[i] instanceof float[]) {
				float[] s1= (float[])s[i];
				System.arraycopy(s1, 0, ((float[])t[i]), 0, s1.length);
			} else {
              if (s[i] instanceof Object) t[i]= s[i];
            }
          }
        }
      } 
    }
  } 
//--------------------------------- c2s ------------------
	/** Converts a character into a String object */
  public static String c2s(char c){
    return (""+c).intern();
  }
//-------------------------- FreeNT ----------------------
  public static void FreeNT(String a) {
	  getToolsInstance().unload(a);
	}
  public static void freeNT(String a) {
	  getToolsInstance().unload(a);
	}
//-------------------------- unload ----------------------
  public void unload(String a) {
		ModLoader.freeNT(a);
		Translator.getCurrentTranslator().resetNT(a);
	}
  
//  Wrapper methods:
  //-------------------------- RegistNT --------------------
  public static NonTerm RegistNT(String a){ 
	  return Translator.getCurrentTranslator().registNT(a);
  }
  //-------------------------- setSyno ---------------------
  public static void setSyno(String nt, String a) {
	  Translator.getCurrentTranslator().setSynonym(nt, a);
  }
  //-------------------------- PushNT ----------------------
  public static void PushNT(String nt, String a) {
	  Translator.getCurrentTranslator().pushNT(nt, a);
  }
  //-------------------------- PopNT -----------------------
  public static void PopNT(String a) { 
	  Translator.getCurrentTranslator().popNT(a);
  }
  //-------------------------- ModNT -----------------------
  public static void ModNT(String nt, String a) { 
	  Translator.getCurrentTranslator().modNT(nt, a);
  }
  //-------------------------- LdMod -----------------------
  public static int LdMod(String a) {
	  return Translator.getCurrentTranslator().LdMod(a);
  }

}