/** Depot4 (c) J. Lampe 1996-2010
Translator.java

		A translator is the shell for use in one thread. It contains
		all the translation specific data which were static in former (pre 1.9)
		versions of Depot4.
    	For logging purposes, a translator can be assigned a name.

 * @author   J. Lampe
 * @version  1.9.4
 * @since 	 JDK 1.4.x
 * $Date: 2010/06/02 $
 */

package Dp4;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

public class Translator {
	
	public NonTermT curNT;
	private String thisMod;
	private final String name;

	// stores Objects
	Object[] translatorProperties= new Object[12];

	protected Glob globalsTable= Glob.getNewGroup();
	protected boolean isOk, isCap= true;

	private HashMap nonTermList= new HashMap();

	private static LogStub log= LogStub.getLog(Translator.class.getName());

	private static List propertyNames= new ArrayList();
	private static ArrayList listeners= new ArrayList();
	
	private static ThreadLocal currentTranslator= new ThreadLocal(){
		protected Object initialValue() {
			return Translator.createTranslator("<default>");
		}
	};

	/**
	 * Constructor
	 * 
	 * @param name	the name of the translator
	 */
	private Translator(String name) {
		this.name= name;
		curNT= registNT("<unknown>");
		log.log(-1, "new Translator "+name);
	}

	/** Returns a unique id for a property name String */
	public synchronized static int getPropertyKey(String name) {
		int result= propertyNames.indexOf(name);
		if (result == -1) {
			result= propertyNames.size();
			propertyNames.add(name);
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "getPropertyKey("+name+")="+result);
		}
		return result;
	}

	/**
	 * Returns the defaultTranslator if no current one is set.
	 */
	public synchronized static Translator getCurrentTranslator() {
    	Object obj= currentTranslator.get();
    	Translator result;
    	if (obj instanceof Translator)
    	    result= (Translator)obj;
    	else {
       		log.log(1, "no current translator found", new Throwable("Stacktrace"));
    	    result= createTranslator();
    	    currentTranslator.set(result);
    	}
    	return result;
	}

	/**
	 * Sets the current translator for the calling thread.
	 */
	public void startTranslation() {
		if (log.isLogEnabled(-1)) {
			log.log(-1, "startTranslation "+name+" isCap="+isCap);
		}
    	Translator obj= (Translator)currentTranslator.get();
    	if (obj!=this) {
    		log.log(-2, "no current translator found", new Throwable("Stacktrace"));
    		currentTranslator.set(this);
    	}
	}

	/**
	 * Removes the current translator for the calling thread.
	 */
	public void endTranslation() {
		if (log.isLogEnabled(-1)) {
			log.log(-1, "endTranslation "+name+" isOk="+isOk);
		}
	}

	/** Handle translator property settings */
	public void setValueFor(int key, Object value) {
		if (translatorProperties.length <= key) {
			Object[] newObj= new Object[key * 2];
			System.arraycopy(translatorProperties, 0, newObj, 0, translatorProperties.length);
			translatorProperties = newObj;
		}
		translatorProperties[key]= value;
		if (log.isLogEnabled(-2)) {
			log.log(-2, "setValueFor("+key+","+value+")");
		}
	}

	public Object getValueFor(int key) {
		Object result= null;
		if (translatorProperties.length > key){
			result= translatorProperties[key];
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "getValueFor("+key+")="+result);
		}
		return result;
	}

	public boolean isOk() {
		return isOk;
	}

	public void setOk(boolean is) {
		isOk= is;
	}

	public boolean isCap() {
		return isCap;
	}

	public void setCap(boolean is) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "setCap("+is+")");
		}
		isCap= is;
	}

	public String getName() {
		return name;
	}

	// ---------------------------- getNT ----------------------------
	NonTerm getNT(String a) {
		NonTerm result= (NonTerm)nonTermList.get(a);
		return result;
	}

	// -------------------------- RegistNT ----------------------------
	public NonTerm RegistNT(String a) {
		return registNT(a);
	}

	public NonTerm registNT(String a) {
		String pack= null;
		String ntName= a;
		int j= a.lastIndexOf('.');
		if (j>0 && j+1<a.length()) {
			pack= a.substring(0, j);
			ntName= a.substring(j+1);
			ModLoader.addPackage(pack);
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "registNT("+ntName+") package="+pack);
		}
		NonTerm nt= getNT(ntName);
		if (nt == null) {
			nt= new NonTerm(this, ntName);
			nonTermList.put(ntName, nt);
		}
		return nt;
	}

	// -------------------------- InstallNT ----------------------------
	public void InstallNT(NonTerm nt, NTprocedure p) {
		if (nt == null || !nt.impname.equals(thisMod))
			return;
		if (nt.mon)
			nt.monP = p;
		else
			nt.P = p;
	}

	public void installNT(NonTerm nt, NTprocedure p) {
		if (nt == null || !nt.impname.equals(thisMod))
			return;
		if (nt.mon)
			nt.monP = p;
		else
			nt.P = p;
	}

	// -------------------------- installNT ----------------------------
	public void installNT(String name, NTprocedure p) {
		Iterator enu= nonTermList.values().iterator();
		NonTerm nt= null;
		while (enu.hasNext()) {
			nt= (NonTerm)enu.next();
			if (nt.eqname.equals(name)) {
				nt.installNT(p);
			}
		}
  }

	// -------------------------- setSyno ----------------------------
	public void setSynonym(String name, String synonym) {
		NonTerm nt;
		nt= RegistNT(name);
		nt.eqname= synonym;
	}

	// -------------------------- ClrNT ----------------------------
	public void clearNT(NonTerm nt) {
		nt.clearNT(this);
	}

	// -------------------------- PushNT ----------------------------
	public void pushNT(String nt, String a) {
		NonTerm r;
		r= registNT(nt);
		r.pushNT(a);
	}

	// -------------------------- PopNT ----------------------------
	public void popNT(String a) {
		NonTerm nt;
		nt= getNT(a);
		if (nt != null) {
			nt.popNT();
		}
	}

	// -------------------------- ModNT ----------------------------
	public void modNT(String nt, String a) {
		NonTerm r;
		r= registNT(nt);
		r.modNT(a);
	}

	// -------------------------- EnumNT ----------------------------
	public void EnumNT(enumProcT ep) {
		Iterator enu= nonTermList.values().iterator();
		while (enu.hasNext()) {
			ep.process(this, (NonTerm) enu.next());
		}
	}

	public void enumerateNT(enumProcT ep) {
		Iterator enu= nonTermList.values().iterator();
		while (enu.hasNext()) {
			ep.process(this, (NonTerm) enu.next());
		}
	}

	// ------------------------------------- LdMod -------------------------------------
	public int LdMod(String n) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "loadModule "+n);
		}
		thisMod= n;
		ModLoader.loadModule(n, this);
		return ModLoader.LdErr;
	}

	public int loadModule(String n) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "loadModule "+n);
		}
		thisMod= n;
		ModLoader.loadModule(n, this);
		return ModLoader.LdErr;
	}

	// ------------------------------------- resetNT -------------------------------------
	public void resetNT(String name) {
		Iterator enu= nonTermList.values().iterator();
		NonTerm nt= null;
		while (enu.hasNext()) {
			nt= (NonTerm) enu.next();
			nt.resetNT(name);
		}
	}

	public Glob getGlob() {
		return globalsTable;
	}
	// ----------------------------- factory methods ---------------------------	
	public static Translator createTranslator(String name) {
		Translator result= new Translator(name);
		fireTranslatorCreated(result);
		return result;
	}
	private static int trcount= 0;
	public static Translator createTranslator() {
		Translator result= new Translator("Translator "+(++trcount));
		fireTranslatorCreated(result);
		return result;
	}

	// ---------------------- handle TranslatorCreationListeners -------------
	public static void addTranslatorCreationListener(TranslatorCreationListener l) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "addTranslatorCreationListener "+l);
		}
		listeners.add(l);
	}

	public static void removeTranslatorCreationListener(TranslatorCreationListener l) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "removeTranslatorCreationListener "+l);
		}
		listeners.remove(l);
	}

//	@SuppressWarnings("unchecked")
	protected static void fireTranslatorCreated(Translator translator) {
		if (log.isLogEnabled(-2)) {
			log.log(-2, "fireTranslatorCreated "+translator);
		}
		currentTranslator.set(translator);
		List clone= (List)listeners.clone();
		for (int i= 0; i < clone.size(); i++) {
			try {
				((TranslatorCreationListener) clone.get(i)).translatorCreated(translator);
			} catch (Exception ex) {
				OP.WrStr(" *** " + ex + "\n");
				if (true || Base.LogLevel.get(13)) {
					OutputStream b= new ByteArrayOutputStream(1000);
					ex.printStackTrace(new PrintStream(b));
					OP.WrStr(b.toString());
				}
				log.log(1, "fireTranslatorCreated", ex);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
//	@Override
	public String toString() {
		return super.toString()+" "+name;
	}
	
	
}