/** Depot4 (c) J. Lampe 1996-2010
	Implements some static utility methods.
	Applies Ml4's naming conventions.

    * @author   Juergen Lampe
    * @version  1.9.4
	* @since 	JDK 1.4.x 
    * $Date: 2010/06/02 $
    */

package Dp4;

import java.io.File;
import java.security.AccessControlException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Util {
	
	private static LogStub log= LogStub.getLog(Util.class.getName());

//------------------------------- id2pack ---------------------------------
	/** Separates a String 'AbcdEfgHi' into 'Abcd.EfgHi', e.g. 'Dp4OP' into 'Dp4.OP'.
	* The first part is called the prefix, the second the suffix of this String.
	* Returns the argument String if no pair <code>(small_letter|digit)capital_letter</code> is found.
	*/
	public static String SplitIdent(String id){
		int j, k= -1;
		for (j= 0; j<id.length();++j) {
			char C= id.charAt(j);
			if (k<0&&(Character.isLowerCase(C)||Character.isDigit(C))) 
				k+= 1;
			else if (Character.isUpperCase(C)&&j>0&&k>-1) 
				break;
		}
		String result= id;
		if (j<id.length()) {
			result= id.substring(0, j)+"."+id.substring(j);
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "SplitIdent("+id+")="+result);
		}
		return result;
	}
	public static String id2pack(String id){
		return SplitIdent(id);
	}
	
	/** Returns the "package part", i.e., the substring before the last dot,
	 * or the empty string if there is not dot at all or the dot is the first or last character
	 * @param str the String to parse
	 * @return the "package" or the empty String
	 */
	public static String PackageFrom(String str) {
		String result= "";
		int j= str.lastIndexOf('.');
		if (j>0 && j+1<str.length()) {
			result= str.substring(0, j);
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "PackageFrom("+str+")="+result);
		}
		return result;		
	}
	public static String getPackage(String str) {
		return PackageFrom(str);
	}

//------------------------------- getPrefix -------------------------------
	/** Returns the prefix of the argument String.
	* The prefix may be the empty String "".
	* @see #SplitIdent(String)
	*/
	public static String PrefixFrom(String str){
		int j, k= -1;
		for (j= 0; j<str.length();++j) {
			char C= str.charAt(j);
			if (k<0&&(Character.isLowerCase(C)||Character.isDigit(C))) 
				k+= 1;
			else if (Character.isUpperCase(C)&&j>0&&k>-1) 
				break;
		}
		String result= "";
		if (j<str.length()) {
			result= str.substring(0, j);
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "PrefixFrom("+str+")="+result);
		}
		return result;	
	}
	public static String getPrefix(String id){
		return PrefixFrom(id);
	}

//------------------------------- getSuffix -------------------------------
	/** Returns the suffix of the argument String.
	* The suffix may be the argument if the prefix is empty.
	* It always holds: s.equals(getPrefix(s)+getSuffix(s));
	* @see #SplitIdent(String)
	*/
	public static String SuffixFrom(String str){
		int j, k= -1;
		for (j= 0; j<str.length();++j) {
			char C= str.charAt(j);
			if (k<0&&(Character.isLowerCase(C)||Character.isDigit(C))) 
				k+= 1;
			else if (Character.isUpperCase(C)&&j>0&&k>-1) 
				break;
		}
		String result= str;
		if (j<str.length()) {
			result= str.substring(j);
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "SuffixFrom("+str+")="+result);
		}
		return result;
	}
	public static String getSuffix(String id){
		return SuffixFrom(id);
	}

//----------------------------- toUpperC ------------------------------
	/** Converts all letters of the String into upper case.
	*/
	public static String UpperCase(String t){
		return t.toUpperCase();
	}
	public static String toUpperC(String t){
		return t.toUpperCase();
	}
//----------------------------- toLowerC ------------------------------
	/** Converts all letters of the String into lower case.
	*/
	public static String LowerCase(String str){
		return str.toLowerCase();
	}
	public static String toLowerC(String t){
		return t.toLowerCase();
	}
//----------------------------- up1letter ------------------------------
	/** Converts the first letter of the String into upper case.
	*/
	public static String StartUpperCase(String str){
		StringBuffer b= new StringBuffer(str);
		if (str.length()>0)
			b.setCharAt(0, Character.toUpperCase(str.charAt(0)));
		final String result= b.toString();
		if (log.isLogEnabled(-2)) {
			log.log(-2, "StartUpperCase("+str+")="+result);
		}
		return result;
	}
	public static String up1letter(String t){
		return StartUpperCase(t);
	}
//---------------------------- low1letter ------------------------------
	/** Converts the first letter of the String into lower case.
	*/
      public static String StartLowerCase(String str){
    	  StringBuffer b= new StringBuffer(str);
    	  if (str.length()>0)
    		  b.setCharAt(0, Character.toLowerCase(str.charAt(0)));
  		final String result= b.toString();
		if (log.isLogEnabled(-2)) {
			log.log(-2, "StartLowerCase("+str+")="+result);
		}
		return result;
      }
      public static String low1letter(String t){
    	  return StartLowerCase(t);
      }
//	---------------------------- indexOf ---------------------------------
	/** 
     * Returns the index within this string of the first occurrence of the
     * specified substring, starting at the specified index.  The integer
     * returned is the smallest value <tt>k</tt> for which:
     * <blockquote><pre>
     *     k &gt;= Math.min(fromIndex, base.length()) && base.startsWith(str, k)
     * </pre></blockquote>
     * If no such value of <i>k</i> exists, then -1 is returned.
     *
     * @param   base        the string to search in.
     * @param   str         the substring for which to search.
     * @param   fromIndex   the index from which to start the search.
     * @return  the index within the string of the first occurrence of the
     *          specified substring, starting at the specified index.
	*/
  	public static int IndexOf(String base, String str, int fromIndex){
  	  final int result= base.indexOf(str, fromIndex);
		if (log.isLogEnabled(-2)) {
			log.log(-2, "IndexOf("+base+","+str+","+fromIndex+")="+result);
		}
  	  return result;
  	}
	public static int indexOf(String t, String s){
		  return t.indexOf(s);
		}
//	---------------------------- substring ---------------------------------
	/** 
	 * @param t
	 * @param first
	 * @param last last<0 means until the end, i.e. 
	 * @return
	 */
	public static String Substring(String t, int first, int last){
		String result= "";
		final int l= t.length();
		if (first<l) {
    		if (last<0 || last>l) {
    			result= t.substring(first);
    		} else if (last<l) {
    			result= t.substring(first, last+1);
    		}
		} else {
			log.log(1, "Substring("+t+","+first+") is undefined, set to \"\"");
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "Substring("+t+","+first+","+last+")="+result);
		}
		return result;
	}
	
	/**
	 * @param key the name of the property to look for
	 * @return the value of the System property or empty String
	 */
	public static String Property(String key) {
		String result= "";
		if (key!=null) {
			String val= null;
			try {
				val= System.getProperty(key);
			} catch (AccessControlException ignore) {
				//do nothing, apply empty string
			}
			if (val != null) {
				result= val;
			}
		}
		if (log.isLogEnabled(-2)) {
			log.log(-2, "Property("+key+")="+result);
		}
		return result;
	}
	public static String getProperty(String key) {
		return Property(key);
	}
	
	/**
	 * @param s
	 * @return
	 */
	public static String DbToCamelCase(String s) {
		boolean nxtCap= false;
		StringBuffer sb= new StringBuffer(s.length());
		for (int i= 0; i<s.length(); i++) {
			char c= s.charAt(i);
			if (c=='_') {
				nxtCap= true;
			} else {
				if (nxtCap) {
					c= Character.toUpperCase(c);
				} else {
					c= Character.toLowerCase(c);
				}
				sb.append(c);
				nxtCap= false;
			}
		}
		final String result= sb.toString();
		if (log.isLogEnabled(-2)) {
			log.log(-2, "DbToCamelCase("+s+")="+result);
		}
		return result;
	}
	
	public static String CamelToDbCase(String s) {
		boolean nxtCap= false;
		StringBuffer sb= new StringBuffer(s.length());
		for (int i= 0; i<s.length(); i++) {
			char c= s.charAt(i);
			char lowC= Character.toLowerCase(c);
			if (c==lowC) {
				nxtCap= true;
			} else {
				if (nxtCap) {
					sb.append('_');
				}
				nxtCap= false;
			}
			sb.append(lowC);
		}
		final String result= sb.toString();
		if (log.isLogEnabled(-2)) {
			log.log(-2, "CamelToDbCase("+s+")="+result);
		}
		return result;
	}
	
	public static String Replace(String s, String old, String nw) {
		final int oldlen= old.length();
		int idx= s.indexOf(old); 
		String result= s;
		StringBuffer sb= new StringBuffer(s.length()+16);
		if (oldlen>0 && idx>-1) {
    		int from= 0;
    	    while (idx>-1) {
    	    	sb.append(s.substring(from, idx));
    	    	sb.append(nw);
    	    	from= oldlen+idx;
    	    	idx= s.indexOf(old, from);	
    		}
    	    if (from<s.length()) {
    	    	sb.append(s.substring(from));
    	    }
    	    result= sb.toString();
		}
		return result;
	}

//--------------------------------- Date ----------------------------------
	/** Returns the current date (in standard format) as String.
	* The format of the date can also be set in property depot4.date.format
	*/
  public static String Date(){
    Date date= new Date();
	String dateFormatString= null;
	try {
		dateFormatString= System.getProperty("depot4.date.format");
	} catch (AccessControlException ignore) {
		//do nothing, apply default
	}
	DateFormat dateFormat= (dateFormatString==null||dateFormatString.length()==0)?
									 DateFormat.getDateInstance():new SimpleDateFormat(dateFormatString);
    return dateFormat.format(date);
  }
//--------------------------------- Time ----------------------------------
	/** Returns the current time (in standard format) as String.
	* The format of the time can also be set in property depot4.time.format
	*/
  public static String Time(){
	  Date date= new Date();
	  String timeFormatString= null;
	  try {
		  timeFormatString= System.getProperty("depot4.time.format");
	  } catch (AccessControlException ignore) {
		  //do nothing, apply default
	}
	  DateFormat dateFormat= (timeFormatString==null||timeFormatString.length()==0)?
			  			DateFormat.getTimeInstance():new SimpleDateFormat(timeFormatString);
	  return dateFormat.format(date);
  }
  
  /** Returns date and/or time in a format defined by formatString
   * @param formatString format definition as defined by java.text.simpleDateFormat
   * @return the formatted datetime
   */
  public static String DateTime(String formatString){
	  Date date= new Date();
	  DateFormat dateFormat= (formatString==null||formatString.length()==0)?
			  DateFormat.getDateTimeInstance(): new SimpleDateFormat(formatString);
	  String result= dateFormat.format(date);
	  if (log.isLogEnabled(-2)) {
		  log.log(-2, "DateTime("+formatString+"):"+result);
	  }
	  return result;
  }

//------------------------------- makeFileDir -----------------------------
  /** Creates a file directory as described by the String.
   * If needed all missing directories on the path are created too.
   * Assumes '/' as separator character.
   * Throws a RuntimeException if a file with this name already exists.
   * @see java.io.File(String)
   * @see boolean java.io.File.isDirectory()
   * @see boolean java.io.File.mkdir()
   */
  public static void MakeFileDir(String id){
	  if (log.isLogEnabled(-2)) {
		  log.log(-2, "MakeFileDir:"+id);
	  }
	  int i2= 1, i3;
	  id= id.replace('/', File.separatorChar);
	  do {
		  i3= id.indexOf(File.separator, i2);
		  if (i3==-1) {
			  i3= id.length();
		  }
		  File dir= new File(id.substring(0, i3));
		  if (dir.exists()&!dir.isDirectory()) {
			  OP.WrStr("** Can't create directory \""+id.substring(0, i3)+"\" - name already in use\n");
			  throw new RuntimeException("Can't create directory "); 
		  }
		  if (!(dir.exists()&dir.isDirectory())) {
			  dir.mkdir();
		  }
		  i2= i3+1;
	  } while (i2<id.length());
  }
  public static void makeFileDir(String id){
	  MakeFileDir(id);
  }
	
}
