/** Depot4 (c) J. Lampe 1996-2010
	 config.java
	 
	 Sets the default configuration for each newly created Translator.
   One can added individual settings in method translatorCreated().

    * @author  Juergen Lampe
    * @version  1.9.4
    * Date:   19.03.2003
*/
    
package Dp4;

import java.util.Vector;

public class config implements TranslatorCreationListener{

	private static String defaultConfigurationFileName;
	private static Vector implementationDefaults= new Vector(),
			mappingDefaults= new Vector();
	
	/** Sets a configuration file name. If the file exists it will be processed
	* every time a new Translator is created.
	* No errors are reported - only valid settings will be applied.
	* (Name is unchanged for downward compatibility, should now better be: setConfigurationFileName)
	*/
	public static void read(String cfn){
		defaultConfigurationFileName= cfn;
	}
	public static void setConfigurationFileName(String cfn){
		defaultConfigurationFileName= cfn;
	}
	
	public static void setImplementation(String ntName, String implName) {
		implementationDefaults.add(new String[] {ntName, implName});
	}
	
	public static void setMapping(String name, String targetName) {
		mappingDefaults.add(new String[] {name, targetName});
	}
	
	/** Implements the actual read functionality now.
	*/
	protected void readDefaultConfigurationFile(Translator translator) {
		if (defaultConfigurationFileName!=null) {
			try {
				translator.setCap(false);
				Files.setIOError(1);
				Tools.translate(translator, Files.IFile(defaultConfigurationFileName), "Dp4RdConfg");
				if (Files.ioStat()==0){
					OP.WrStr(Messages.systemMessages[18]+defaultConfigurationFileName+"\n");
				} else {
					OP.WrStr(Messages.systemMessages[15]+defaultConfigurationFileName+"\n");
				}
				Files.setIOError(0);
			}
			catch (Exception ioex) {
				OP.WrStr(Messages.systemMessages[15]+defaultConfigurationFileName+"\n");
			}
		}
	}
	
	protected void processDefaults(Translator translator) {
		String[] b;
		for (int i= 0; i<implementationDefaults.size(); i++) {
			b=(String[])implementationDefaults.get(i);
			translator.modNT(b[0], b[1]);
		}
		for (int i= 0; i<mappingDefaults.size(); i++) {
			b=(String[])mappingDefaults.get(i);
			Mapper.setMap(translator, b[0], b[1]);
		}
	}
 
 	/** Implements the TranslatorCreationListener interface.
	* Sets the defaults for this translator.
	*/
	public void translatorCreated(Translator translator) {
		// set defaults:
		translator.modNT("ident", "Dp4.id");
		translator.modNT("num", "Dp4.num");
		translator.modNT("integer", "Dp4.intn");
		translator.modNT("int", "Dp4.intn");
		translator.modNT("string", "Dp4.str");

		Mapper.setMap(translator, "Dp4OP", "Dp4.OP");
		Mapper.setMap(translator, "Dp4Util", "Dp4.Util");
		Mapper.setMap(translator, "Dp4List", "Dp4.List");
		Mapper.setMap(translator, "Dp4Tools", "Dp4.Tools");
		Mapper.setMap(translator, "Dp4Map", "Dp4.Mapper");
		Mapper.setMap(translator, "Dp4Mapper", "Dp4.Mapper");
		Mapper.setMap(translator, "Dp4Chars", "Dp4.Chars");
		Mapper.setMap(translator, "Dp4Base", "Dp4.Base");
		Mapper.setMap(translator, "Dp4Stdlex", "Dp4.Stdlex");
		Mapper.setMap(translator, "Dp4Messages", "Dp4.Messages");
		Mapper.setMap(translator, "Dp4Streams", "Dp4.Streams");
		Mapper.setMap(translator, "Dp4Files", "Dp4.Files");
		Mapper.setMap(translator, "Dp4StrBuf", "Dp4.StrBuf");
		Mapper.setMap(translator, "Dp4StringStream", "Dp4.StringStream");
		Mapper.setMap(translator, "Dp4DocFiles", "Dp4.DocFiles");
		Mapper.setMap(translator, "Dp4Log", "Dp4.LogStub");
		Mapper.setMap(translator, "Dp4IndStream", "Dp4x.indStreamTrg");
		Mapper.setMap(translator, "Dp4Compile", "Dp4.Compile");
		Mapper.setMap(translator, "Dp4FileTrg", "Dp4.FileTrg");
		Mapper.setMap(translator, "Dp4ModLoader", "Dp4.ModLoader");

		processDefaults(translator);

		readDefaultConfigurationFile(translator);
	}
	
	static {
		config instance= new config();
		Translator.addTranslatorCreationListener(instance);
  }
}
