/** Depot4 (c) J. Lampe 1996-2010

 * @author	Juergen Lampe
 * @version  1.9.4
 * Date:		11.06.2003
 */

package Dp4;

/** Implements id and ident for a special request.
 *
 *	Extended identifier implementation. Accepts identifiers starting
 *	with a letter or underscore followed by a number of letters or digits or underscores and converts
 * them into valid Ml4 identifiers by removing underscores. Leading and trailing underscores are
 * replaced by 'X'. The letter after a (removed) underscore is always converted to upper case.
 * Targets
 *  0 and 1: the converted identifier string (target 0 is cut to the number of valid chars)
 *  2: the original string
 *	
 * Maximal MAX_SIG characters (i.e. 256) are significant.
 */

import java.util.Stack;

public class identToMl4 extends NTprocedure {

	/** The kind of conversion, which is used to get a canonical form of identifiers if
	 * capitalisation is irrelevant.
	 */
	public static final int ALLTOUPPERCASE= 1, ALLTOLOWERCASE= 2;

	private static int exclusionListPropertyKey= Translator.getPropertyKey("identToMl4.XList");
	private static int exclusionListStackPropertyKey= Translator.getPropertyKey("identToMl4.XListStack");

	private static final int MAX_SIG= 256;

	private Translator y_translator;
	private char[] ac;

	protected int conversion= ALLTOUPPERCASE;
	protected int validChars= -1;  // means: all

	//------------------------ id ------------------------
	private trgts y_p_id(NonTermT y_NT, Prs y_Prs){
		String y_SYM, y_SYM1, source_SYM;
		char C;
		int i= 0;
		trgts y_0tg= new trgts();
		NonTermT y_yNT= y_translator.curNT; y_translator.curNT= y_NT;
		y_Prs.Skip();
		C= y_Prs.AC();
		y_Prs.A= isStartCharacter(C);
		if (y_Prs.A) {
			y_Prs.acc(); 
			ac[i++]= C;
			while (isFollowCharacter(C= y_Prs.AC())&&(C!='\000')) {
				y_Prs.acc(); 
				if (i<MAX_SIG)
					ac[i++]= C;
			}
		}
		if (y_Prs.A) {
			source_SYM= String.copyValueOf(ac, 0, i);
			if (ac[0]=='_')
				ac[0]= 'X';
			final int acLen= i+1;
			i= -1;
			for (int j = 0; j<acLen; j++) {
				char c = ac[j];
				i++;
				if (c=='_'){
					if (j<acLen-1) {
						j++;
						c = Character.toUpperCase(ac[j]);
					} else {
						c= 'X';
					}
				}
				ac[i]= c;
			}
			y_SYM= String.copyValueOf(ac, 0, i);
			y_SYM1= y_SYM;
			if (validChars>0)
				y_SYM= y_SYM.substring(0, Math.min(validChars, y_SYM.length()));
			if (!y_translator.isCap()){
				y_0tg.trg0= conversion==ALLTOUPPERCASE?y_SYM.toUpperCase():y_SYM.toLowerCase();
			} else
				y_0tg.trg0= y_SYM;
			if (getLexemExclusionList(y_translator).contains(source_SYM))
				y_Prs.A= false;
			else {
				y_0tg.trg1= Chars.appStr(null, y_SYM1);
				y_0tg.trg2= Chars.appStr(null, source_SYM);
			}
		}
		y_translator.curNT= y_yNT;
		return y_0tg;
	}

	public void init(Translator translator){
		identToMl4 y_ntp_= new identToMl4();
		translator.installNT("id", y_ntp_);
		translator.installNT("ident", y_ntp_);
		translator.installNT("identToMl4", y_ntp_);
		y_ntp_.y_translator= translator;
	}

	public trgts parse(NonTermT NT, Prs Prs){
		return y_p_id(NT, Prs);
	}

	//---------------------- Overwrite these methods in subclass ----------------------
	public identToMl4(){
		ac= new char[MAX_SIG];
	}


	/** Returns true if c is a valid start (first) character
	 */
	protected boolean isStartCharacter(char c) {
		return c=='_' || Character.isLetter(c);
	}

	/** Returns true if c is a valid tail (all but the first) character
	 */
	protected boolean isFollowCharacter(char c) {
		return c=='_' || Character.isLetterOrDigit(c);
	}
	//------------------------- end of methods to overwrite ---------------------------

	private static AccTab getLexemExclusionList(Translator translator) {
		AccTab exclusionList=	(AccTab)translator.getValueFor(exclusionListPropertyKey);
		if (exclusionList==null) {
			exclusionList= new AccTab();
			translator.setValueFor(exclusionListPropertyKey, exclusionList);
		}
		return exclusionList;
	}

	public static AccTab getLexemExclusionList() {
		return getLexemExclusionList(Translator.getCurrentTranslator());
	}

	public static void addToLexemExclusionList(String lexem) {
		getLexemExclusionList(Translator.getCurrentTranslator()).addString(lexem);
	}

	public static void setLexemExclusionListFile(String filename) {
		Translator translator= Translator.getCurrentTranslator();
		AccTab exclusionList=	new AccTab();
		exclusionList.addFromFile(filename);
		translator.setValueFor(exclusionListPropertyKey, exclusionList);
		if (Base.LogLevel.get(10)) 
			OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[8]+"identToMl4.setLexemExclusionListFile:"+filename+";\n");
	}

	public static void pushLexemExclusionListFile(String filename){
		Translator translator= Translator.getCurrentTranslator();
		Stack stack= (Stack)translator.getValueFor(exclusionListStackPropertyKey);
		if (stack==null) {
			stack= new Stack();
			translator.setValueFor(exclusionListStackPropertyKey, stack);
		}
		stack.push(getLexemExclusionList(translator));
		setLexemExclusionListFile(filename);
		if (Base.LogLevel.get(10)) 
			OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[8]+"identToMl4.pushLexemExclusionList:"+filename+";\n");
	}

	public static void popLexemExclusionList(){
		Translator translator= Translator.getCurrentTranslator();
		Stack stack= (Stack)translator.getValueFor(exclusionListStackPropertyKey);
		if (stack==null) {
			return;
		}
		Object obj= stack.pop();
		if (obj!=null) {
			translator.setValueFor(exclusionListStackPropertyKey, obj);
		}
		if (Base.LogLevel.get(10)) 
			OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[8]+"identToMl4.popLexemExclusionList\n");
	}

}
