/** Depot4 (c) J. Lampe 1996-2010

	* @author	Juergen Lampe
	* @version  1.9.4
	* Date:		11.06.2003
	*/

package Dp4;

/** Implements id and ident such that it may easily be subclassed for special requests.
* @see identJava identJava for an example (Java style identifiers)
*
*	Extended identifier implementation. Accepts identifiers starting
*	with a letter followed by a number of letters or digits or underscores.
*	
* Maximal MAX_SIG characters (i.e. 256) are significant.
*/

import java.util.Stack;

public class identX extends NTprocedure {

  /** The kind of conversion, which is used to get a canonical form of identifiers if
   * capitalization is irrelevant.
   */
  public static final int ALLTOUPPERCASE= 1, ALLTOLOWERCASE= 2;

	private static int exclusionListPropertyKey= Translator.getPropertyKey("identX.XList");
	private static int exclusionListStackPropertyKey= Translator.getPropertyKey("identX.XListStack");

	private Translator y_translator;

	protected int conversion= ALLTOUPPERCASE;
	protected int validChars= -1;  // means: all

	//------------------------ id ------------------------
  private trgts y_p_id(NonTermT y_NT, Prs y_Prs){
	  String y_SYM, y_SYM1;
	  char C;
	  StringBuffer sb= new StringBuffer();
	  trgts y_0tg= new trgts();
	  NonTermT y_yNT= y_translator.curNT; y_translator.curNT= y_NT;
	  y_Prs.Skip();
	  C= y_Prs.AC();
	  y_Prs.A= isStartCharacter(C);
	  if (y_Prs.A) {
		  y_Prs.acc(); 
		  sb.append(C);
		  while (isFollowCharacter(C= y_Prs.AC()) && (C!='\000')) {
			  y_Prs.acc(); 
			  sb.append(C);
		  }
	  }
	  if (y_Prs.A) {
		  y_SYM= sb.toString();
		  y_SYM1= y_SYM;
		  if (validChars>0) {
			  y_SYM= y_SYM.substring(0, Math.min(validChars, y_SYM.length()));
		  }
		  if (!y_translator.isCap()){
			  y_0tg.trg0= conversion==ALLTOUPPERCASE ? y_SYM.toUpperCase() : y_SYM.toLowerCase();
		  } else {
			  y_0tg.trg0= y_SYM;
		  }
		  if (getLexemExclusionList(y_translator).contains(y_0tg.trg0)) {
			  y_Prs.A= false;
		  } else {
			  y_0tg.trg1= Chars.appStr(null, y_SYM1);
		  }
	  }
	  y_translator.curNT= y_yNT;
	  return y_0tg;
  }

	public void init(Translator translator){
		identX y_ntp_= getNew(0);
		translator.installNT("id", y_ntp_);
		translator.installNT("ident", y_ntp_);
		String name= getClass().getName();
		name= name.substring(name.lastIndexOf('.')+1);
		translator.installNT(name, y_ntp_);
		y_ntp_.y_translator= translator;
	}

	private final int m;
	public trgts parse(NonTermT NT, Prs Prs){
	  return y_p_id(NT, Prs);
	}
	
//---------------------- Overwrite these methods in subclass ----------------------
	/**Constructor */
	protected identX(int i){
		this.m= i;
	}

	public identX(){
		m= -1; // create dummy instance
	}
	
	/** get a new instance of the actual class
	*/
	protected identX getNew(int i) {
		return new identX(i);  //!replace identX by actual class!!!
	}
	
	/** Returns true if c is a valid start (first) character
	*/
	protected boolean isStartCharacter(char c) {
		return Character.isLetter(c);
	}
	
	/** Returns true if c is a valid tail (all but the first) character
	*/
	protected boolean isFollowCharacter(char c) {
		return c=='_' || Character.isLetterOrDigit(c);
	}
//------------------------- end of methods to overwrite ---------------------------
	
	private static AccTab getLexemExclusionList(Translator translator) {
		AccTab exclusionList= (AccTab)translator.getValueFor(exclusionListPropertyKey);
		if (exclusionList==null) {
			exclusionList= new AccTab();
			translator.setValueFor(exclusionListPropertyKey, exclusionList);
		}
		return exclusionList;
	}
	
	public static AccTab getLexemExclusionList() {
		return getLexemExclusionList(Translator.getCurrentTranslator());
	}
	
	public static void addToLexemExclusionList(String lexem) {
		getLexemExclusionList(Translator.getCurrentTranslator()).addString(lexem);
	}
	
	public static void setLexemExclusionListFile(String filename) {
		Translator translator= Translator.getCurrentTranslator();
		AccTab exclusionList= new AccTab();
		exclusionList.addFromFile(filename);
		translator.setValueFor(exclusionListPropertyKey, exclusionList);
		if (Base.LogLevel.get(10)) 
			OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[8]+"idX.setLexemExclusionListFile:"+filename+";\n");
	}
	
	public static void pushLexemExclusionListFile(String filename){
		Translator translator= Translator.getCurrentTranslator();
		Stack stack= (Stack)translator.getValueFor(exclusionListStackPropertyKey);
		if (stack==null) {
			stack= new Stack();
			translator.setValueFor(exclusionListStackPropertyKey, stack);
		}
		stack.push(getLexemExclusionList(translator));
		setLexemExclusionListFile(filename);
		if (Base.LogLevel.get(10)) 
			OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[8]+"idX.pushLexemExclusionList:"+filename+";\n");
	}
	
	public static void popLexemExclusionList(){
		Translator translator= Translator.getCurrentTranslator();
		Stack stack= (Stack)translator.getValueFor(exclusionListStackPropertyKey);
		if (stack==null) {
			return;
		}
		Object obj= stack.pop();
		if (obj!=null) {
			translator.setValueFor(exclusionListStackPropertyKey, obj);
		}
		if (Base.LogLevel.get(10)) 
			OP.WrStr(Messages.systemMessages[7]+Messages.systemMessages[8]+"idX.popLexemExclusionList\n");
	}
	
}
