/** Depot4 (c) J. Lampe 1996-2010
    tNode.java
		
    The base class of all nodes in the target tree.
		tNodes implement in some sense a generlized String. These 'tNode strings'
		are identified by their first node.
		This is a simple straight forward implementation. One could think about
		caching the currently active (i.e. last) node but tests within the translator
		indicated no reasonable gain as the chains tend to be short.
    
	* @author	Juergen Lampe
	* @version  1.9.4
	* Date:		24.04.2003
  */
    
package Dp4;

/** Base class for target tokens.
 *  Just the container to link them into a chain.
 * 
 * @author lampe
 *
 */
public class tNode {
	
	private static LogStub Log= LogStub.getLog(tNode.class.getName());

	/** Next token of type tNode in target chain */
	protected tNode nxt= null;

	/** Constructor */
	public tNode(){
	}
	
	public void invokeHandler(Target m) {
		m.Handler(this);
	}

	/** Returns the last one in the chain of tNodes appended to the argument.
	 */
	public static tNode thistNd(final tNode n) {
		tNode h= n;
		while (h.nxt!=null) {
			h= h.nxt; 
		}
		return h;
	}


	/** Appends a tNode to a chain of nodes.
	 * @param t  the chain of nodes (if null, a becomes the root of the chain)
	 * @param a  the node to append
	 * @return t (or a if t is null)
	 */
	public static tNode apptN(tNode t, tNode a) {
		if (t==null){ 
			t= a;
		} else { 
			tNode h= thistNd(t); 
			h.nxt= a;
		}
		return t;
	}
	
	/** Appends a chain of tNodes with a the root of this chain.
	 *  In difference to apptNode, a is not inlined into the chain but a special type of tNode (ptNode) is
	 *  appended that has a as a member, thus branching the chain.
	 * @param t
	 * @param a
	 * @return t
	 */
	public static tNode appChn(tNode t, tNode a) {
		if (Log.isLogEnabled(-2)) {
			Log.log(-2, "appChn "+t+"/"+a);
		}
		if (a!=null) { 
			ptNode n= new ptNode(a); 
			t= apptN(t, n);
		}
		if (a==trgts.TrgNotDef) {
			OP.WrStr(((NonTerm)Translator.getCurrentTranslator().curNT).name+": ");
			OP.ERROR(17);
		}
		return t;
	}

	/** Appends a (source) token to the target chain
	 * @param t target chain's root
	 * @param T token to include
	 * @return t
	 */
	public static tNode appTkn(tNode t, Token T) {
		if (Log.isLogEnabled(-2)) {
			Log.log(-2, "appTkn "+t+"/"+T);
		}
		if (T!=null) { 
			TkntNode n= new TkntNode(T); 
			t= apptN(t, n);
		}
		return t;
	}

	/** Deep first tree walker, invoking the handler method for all tNodes of the tree
	 * @param t root of the tNode tree to walk through
	 * @param m argument to propagate to each of the handler calls
	 */
	public static void alltNodes(tNode t, Target m) {
		tNode h= t;
		while (h!=null) {
			h.invokeHandler(m);
			h= h.nxt;
		}
	}

}
