/** Depot4 (c) J. Lampe 2002-2003
 *
 * The default implementation of an AST node.
 * Stores the targets object of the associated nonterminal invocation.
 *
 * @author  Juergen Lampe
 * @version 1.9.2
 * date:   17.03.2003
 */

package Dp4x;

import java.util.List;
import java.util.ArrayList;
import Dp4.trgts;

public class ASTDefaultNode implements ASTNode {

	protected List children= new ArrayList();
	protected trgts targets;
	private final String name;

	/** Constructor
	 * @param name the name of the associated rule.
	 */
	public ASTDefaultNode(String name) {
		this.name = name;
	}

	/** Returns the name of the nonterminal */
	public String getName() {
		return name;
	}

	/** Adds an ASTNode as child node */
	public void addChild(ASTNode child) {
		children.add(child);
	}

	/** Retruns whether this node has children */
	public boolean hasChildren() {
		return children.size()>0;
	}

	/** Returns the children of this node */
	public List getChildren() {
		return children;
	}

	/** Returns the targets as returned from the associated rule invocation.
	 * @return the targets
	 */
	public trgts getTargets() {
		return targets;
	}

	/** Sets the targets for this AST node. (Call only during build of the AST!)
	 * @param targets
	 */
	public void setTargets(trgts targets) {
		this.targets = targets;
	}

	/** Returns a string representation of the object.
	 * @return the name of the associated rule
	 */
	public String toString() {
		return name;
	}
}
