/** Depot4 for Java (c) J. Lampe 1997-2003
 STree.java
 This is a wrapper for all nonterminal invocations, collecting parsing data.
 Prints some statistics and a simplified syntax tree.

* @author  Juergen Lampe
* @version 1.9.2
* date:   17.03.2003
*/

package Dp4x;

import Dp4.*;

public class STree extends NTprocMonT {

	int currentCallLevel = 0, maxCallLevel = 0;
	int stackPointer = 0;
	int maxNTId = 0, ntIdStackSize = 2048;
	int[] ntIdStack = new int[ntIdStackSize];
	STreeNTMap streeList = new STreeNTMap(this);
	private boolean isNotBPObserving = true;
	private Translator translator;

	/** Initializes the data structures */
	public void clear() {
		currentCallLevel = 0;
		maxCallLevel = 0;
		stackPointer = 0;
		maxNTId = 0;
		isNotBPObserving = true;
	}

//----------------------- parse -----------------------
	/** Overwrite superclass method to hande invoking and exiting
	 * parsing method.
	 * @param nonTerm
	 * @param p
	 * @return the propagated targets
	 */
	public trgts parse(NonTermT nonTerm, Prs p) {
		trgts h;
		if (isNotBPObserving) {
			Prs.addBrPtObs(p, new STreeBP(this));
			isNotBPObserving = false;
		}
		int currentNTId = ((NonTerm)nonTerm).id;
		if (currentNTId>maxNTId)
			maxNTId = currentNTId;
		currentCallLevel++;
		if (currentCallLevel>maxCallLevel)
			maxCallLevel = currentCallLevel;
		if (stackPointer>=ntIdStackSize) {
			ntIdStackSize += ntIdStackSize;
			ntIdStack = resize(ntIdStackSize, ntIdStack);
		}
		ntIdStack[stackPointer++] = currentNTId;

		h = super.parse(nonTerm, p);

		currentCallLevel--;
		if (stackPointer>=ntIdStackSize) {
			ntIdStackSize += ntIdStackSize;
			ntIdStack = resize(ntIdStackSize, ntIdStack);
		}
		if (ntIdStack[stackPointer-1]<0)
			ntIdStack[stackPointer-1]--;
		else
			ntIdStack[stackPointer++] = -1;
		return h;
	}
//----------------------- List -------------------------
	/** Prints the results to the log.
	 */
	public void List() {
		int l = 0, i, j, nc = 0;
		for (i = 0; i<stackPointer; i++) {
			if (ntIdStack[i]>=0) {
				nc++;
			}
		}
		Dp4.OP.WrStr("Counts\n------\nmax. tree heigth:  "
								 +maxCallLevel
								 +"\nno. of nodes:  "
								 +nc
								 +"\n======================\n");
		translator.enumerateNT(streeList);
		for (i = 0; i<stackPointer; i++) {
			if (ntIdStack[i]<0)
				l += ntIdStack[i];
			else {
				for (j = 0; j<=l; j++)
					Dp4.OP.WrStr(". ");
				Dp4.OP.WrStr(streeList.getName(ntIdStack[i])+"\n");
				l++;
			}
		}
	}

	/** Utility method for increasing the integer array */
	public static int[] resize(int ns, int[] old) {
		int[] nw = new int[ns];
		System.arraycopy(old, 0, nw, 0, old.length);
		return nw;
	}

	public STree(Translator translator) {
		this.translator = translator;
	}
}

/* ------------------------------------------------------- */
/** Helper class, builds the map from id to name of NonTerms */
class STreeNTMap extends enumProcT {
	private STree currentSTree;
	private String[] idToNameMap = null;

	STreeNTMap(STree sTree) {
		currentSTree = sTree;
		idToNameMap = new String[currentSTree.maxNTId+1];
	}

	public void process(Translator translator, NonTermT n) {
		int currentNTId = ((NonTerm)n).id;
		if (currentNTId<=currentSTree.maxNTId) {
			idToNameMap[currentNTId] = ((NonTerm)n).name;
		}
	}

	public String getName(int id){
		return idToNameMap[id];
	}

}

/* ------------------------------------------------------- */
/** A branch point observer */
class STreeBP extends BrPtObs {
	private STree ct;
	private int p = -1, mst = 64;
	private int[] st = new int[mst], cl = new int[mst];

	//----------------------------- ND ------------------------
	public void ND(Dp4.Prs0 P) {
		super.ND(P);
		p = -1;
	}

	//----------------------------- BT ------------------------
	public void BT(Dp4.Prs0 P) {
		super.BT(P);
		if (p>=0) {
			ct.stackPointer = st[p];
			ct.maxCallLevel = cl[p];
		}
	}

	//----------------------------- DB ------------------------
	public void DB(Dp4.Prs0 P) {
		super.DB(P);
		if (p>-1) p--;
	}

	//----------------------------- SB ------------------------
	public void SB(Dp4.Prs0 P) {
		super.SB(P);
		if (p>=mst-1) {
			mst += mst;
			st = STree.resize(mst, st);
			cl = STree.resize(mst, cl);
		}
		st[++p] = ct.stackPointer;
		cl[p] = ct.maxCallLevel;
	}

	STreeBP(STree c) {
		ct = c;
	}
}
