/** Depot4 IDE (c) J. Lampe 1999-2010
    AboutWindow.java
    
    * @author	Juergen Lampe
    * @version 1.3.4
    * $Date.$
    * @since JDK 1.2
    */
    
package Dp4.ide;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.text.DecimalFormat;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;

import Dp4.Version;

public final class AboutWindow extends JPanel {

	private static final long serialVersionUID= -4599434046372214284L;

	private static final DecimalFormat decimalFormat= new DecimalFormat("####,###");

	private static final String[] txt= {"Depot4 for Java",
		MainData.IdeVersion+" of "+MainData.VersionDate,
		"Runtime "+Version.getVersionString()+" of "+Version.getVersionDateString(),
		"by J. Lampe",
		"ju_lampe@web.de",
		"(c) J. Lampe 1997-2010", "     ",
		"For more information/help see:",
		"http://www.tools.vlampe.de/depot4"
	};
	String sysinf;
	JTextArea ta= new JTextArea(8, 40);

	public void setVisible(boolean v){
		if (v) {
			String stinf= "";
			if (MainData.getIt().isStandalone) {
				long totalMemory = Runtime.getRuntime().totalMemory();
				long usedMemory= totalMemory - Runtime.getRuntime().freeMemory();
				stinf=
					"Total memory "+decimalFormat.format(totalMemory)+
					"\nUsed memory  "+decimalFormat.format(usedMemory)+
					"  ("+(100*usedMemory/totalMemory)+"%)";
			}
			ta.setText(sysinf+stinf);
			ta.validate();
		}
		super.setVisible(v);
	}

	public AboutWindow(){
		JPanel p3= new JPanel(), p4= new JPanel(), p5= new JPanel();
		sysinf= "Java Version "+System.getProperty("java.version")+
		"\n"+System.getProperty("os.name")+
		" "+System.getProperty("os.version");
		if (MainData.getIt().isStandalone) {
			StringBuffer cp= new StringBuffer(System.getProperty("java.class.path"));
			for (int i= 0; i<cp.length(); i++) {
				if (cp.charAt(i)== File.pathSeparatorChar) {
					cp.insert(i+1, "\n           ");
				}
			}
			sysinf+="\nCLASSPATH= "+cp+"\nDirectory= "+System.getProperty("user.dir")+"\n";
		}
		p3.setLayout(new BoxLayout(p3, BoxLayout.Y_AXIS));
		p3.setBackground(Color.yellow);
		JLabel l= new JLabel(txt[0], SwingConstants.CENTER);
		l.setFont(new Font("SansSerif", Font.BOLD+Font.ITALIC, l.getFont().getSize()*2));
		l.setForeground(Color.black);
		l.setAlignmentX(Component.CENTER_ALIGNMENT);
		p3.add(l);
		p3.add(Box.createRigidArea(new Dimension(10, 15)));
		for (int i= 1; i<txt.length;i++) {
			l= new JLabel(txt[i], SwingConstants.CENTER);
			l.setAlignmentX(Component.CENTER_ALIGNMENT);
			p3.add(l);
		}
		p3.setBorder(BorderFactory.createEmptyBorder(20, 40, 10, 40));
		p5.setLayout(new FlowLayout());
		l= new JLabel("System information:", SwingConstants.LEFT);
		l.setFont(new Font("SansSerif", Font.BOLD, l.getFont().getSize()+4));
		l.setForeground(Color.black);
		l.setAlignmentX(Component.LEFT_ALIGNMENT);
		p4.setLayout(new BoxLayout(p4, BoxLayout.Y_AXIS));
		p4.add(l);
		p4.add(Box.createRigidArea(new Dimension(10, 10)));
		ta.setAlignmentX(Component.LEFT_ALIGNMENT);
		p5.add(p4);
		p5.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
		p4.add(ta);
		setLayout(new BorderLayout(2,20));
		add(p3, BorderLayout.NORTH);
		add(new JScrollPane(p5), BorderLayout.CENTER);
		ta.setFont(new Font("DialogInput", Font.PLAIN, ta.getFont().getSize()));
		ta.setEditable(false);
		ta.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 10));
		ta.setBackground(getBackground());
		setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
	}
	// implement IdeTab
	private boolean isontop= false;
	private TextDisplay textDisplay;
	private JMenuBar menubar= MainData.getIt().menubar;

	public void setOnTop(){
		if (isontop) return;
		isontop= true;
		if (textDisplay!=null&&textDisplay instanceof FocusListener)
			((FocusListener)textDisplay).focusGained(null);
		//updateMenu();
		setVisible(true);
		menubar.getMenu(MainData.TRANSLATE_MENU).setEnabled(false);
		menubar.getMenu(MainData.OPTIONS_MENU).setEnabled(false);
	}
	public void setBack(){
		if (!isontop) return;
		isontop= false;
		textDisplay= MainData.getIt().textDisplay;
		//updateData();
		setVisible(false);
		menubar.getMenu(MainData.TRANSLATE_MENU).setEnabled(true);
		menubar.getMenu(MainData.OPTIONS_MENU).setEnabled(true);
	}
	public boolean isOnTop(){
		return isontop;
	}

	static JFrame getAboutFrame() {
		final JFrame frame= new JFrame("About");
		AboutWindow about= new AboutWindow();
		frame.getContentPane().add(about);
		about.setVisible(true);
		frame.pack();
		Rectangle bounds= MainData.getIt().frame.getBounds();
		Dimension dim= frame.getPreferredSize();
		frame.setLocation(bounds.x+Math.max(0, (bounds.width-dim.width)/2),
				bounds.y+Math.max(0, (bounds.height-dim.height)/2));
		frame.addWindowListener(
				new WindowAdapter() {
					public void windowClosing(WindowEvent e) {
						frame.setVisible(false);
						frame.dispose();
					}
				}
		);
		return frame;
	}

}