/** Depot4 IDE (c) J. Lampe 1999-2003
    AbstractApplicationPanel.java

    * @author  Juergen Lampe
    * @version 1.3.3
    * @since	 JDK 1.2
    * Date:   20.03.2003
    */
    
package Dp4.ide;

import java.awt.event.FocusListener;
import java.awt.Font;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JPanel;

import Dp4.OP;

public abstract class AbstractApplicationPanel extends JPanel implements IdeTab, Runnable {

  public String id, tip;
  TextDisplayElement src;
  protected TextDisplayElement optionalSecondTextDisplay;
  boolean isCase= true, isCheck= false, isTrace= false,
  		 isSelectionOnly= false, isWarnings= false, isVerboseCompiling= false;
  protected boolean isUseWin;
  
  private boolean isontop= true;
  private TextDisplay textDisplay;

	public AbstractApplicationPanel() {
		id= getInstanceName();
  }

	public void Do() {
		if (isSelectionOnly && !src.isSelectionNotEmpty()) {
			OP.WrStr("*** Nothing selected ***\n");
			return;
		}
		(new Thread(this)).start();
	}

	protected abstract String getInstanceName();

  void updateMenu() {
		((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.CASE]).setState(isCase);
		((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.CHECK]).setState(isCheck);
		((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.TRACE]).setState(isTrace);
		((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.WARNINGS]).setState(isWarnings);
		((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.VERBOSE]).setState(isVerboseCompiling);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.CASE].setEnabled(isUseWin);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.CHECK].setEnabled(isUseWin);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.TRACE].setEnabled(isUseWin);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.WARNINGS].setEnabled(!isUseWin);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.VERBOSE].setEnabled(!isUseWin);
  }
  
  void updateData() {
		isCase= ((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.CASE]).getState();
		isCheck= ((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.CHECK]).getState();
		isTrace= ((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.TRACE]).getState();
 		isWarnings= ((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.WARNINGS]).getState();
 		isVerboseCompiling= ((JCheckBoxMenuItem)MainData.menuItems[MainData.OPTIONS_MENU][MainData.VERBOSE]).getState();
  }
  
  // implement IdeTab
  
	public void setOnTop(){
		isontop= true;
		if (textDisplay!=null&&textDisplay instanceof FocusListener)
			((FocusListener)textDisplay).focusGained(null);
		else if (textDisplay==null) {
			MainData.getIt().switchTextDisplay(null);
		}
		updateMenu();
	}
  	
	public void setBack(){
  		if (!isontop) return;
		isontop= false;
		textDisplay= MainData.getIt().getActiveTextDisplay();
		updateData();
	}
	
	public boolean isOnTop(){
		return isontop;
	}

	public String getFontDesc(){
		String fontDsc = getFontDsc(src.getFont());
		if (optionalSecondTextDisplay!=null){
			fontDsc+= ","+getFontDsc(optionalSecondTextDisplay.getFont());
		}
		return fontDsc;
	}

	public void setFontDesc(int no, String fontDsc){
		if (no==0)
		  src.setFont(Font.decode(fontDsc));
		else if (no==1 && optionalSecondTextDisplay!=null)
      optionalSecondTextDisplay.setFont(Font.decode(fontDsc));
	}


 /****************************************************************************************/
 // Convenience methods
 /****************************************************************************************/

  protected static final String[] styles= {"PLAIN", "BOLD", "ITALIC", "BOLDITALIC"};

  /** Return a string representation of the given font.
   * The return value is null (if argument null) or a string of the following form:
   * <ul>
   * <li><em>fontname-style-pointsize</em>
   * </ul>
   * where <i>style</i> is one of the three strings
   * <code>"BOLD"</code>, <code>"BOLDITALIC"</code>, or
   * <code>"ITALIC"</code>, and point size is a decimal
   * representation of the point size.
   * <p>
   */
  public static String getFontDsc(Font font) {
  	if (font==null)
  		return null;
  	else {
  		return font.getName()+"-"
  			+styles[font.getStyle()]+"-"
  			+font.getSize();
  	}
  }

}