/** Depot4 IDE (c) J. Lampe 1999-2003
 *
 * @author  Juergen Lampe
 * @version 1.3.3
 * date:   17.03.2003
 */

package Dp4.ide;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Stack;
import javax.swing.*;

import Dp4.*;

public class Debug extends UsePanel implements Runnable{

	private static int count= 0;

	protected static Debug currentDebugWindow;

  boolean susp= false;
  Thread par;
  int maxbst= 2;
  Exception ex;
  Stack cstack= new Stack();
  protected DebugNTMonitorState [] bst= {null, null};
  protected DebugBPDefFrame defFrame;
	protected DebugBreakpointFrame breakpointFrame;

	/** Create new debugWindows by use of Debug.New() */
	private Debug(){
		tip= "Debug your translator";
		defFrame= new DebugBPDefFrame("Breakpoints", this);
		breakpointFrame= new DebugBreakpointFrame(this);
  }

	protected String getInstanceName(){
		if (count++>0)
			return "Debug "+count;
		else
			return "Debug";
	}

	public DebugBPDefFrame getDefFrame() {
		return defFrame;
	}

	void updateMenu() {
		super.updateMenu();
		MainData.getIt().menubar.getMenu(MainData.DEBUG_MENU).setEnabled(true);
		currentDebugWindow= this;
	}
	
	public void setBack() {
		super.setBack();
		MainData.getIt().menubar.getMenu(MainData.DEBUG_MENU).setEnabled(false);
		currentDebugWindow= null;
	}

	public void add(String nt, boolean in, boolean out, int ct){
    int i= getTranslator().registNT(nt).id;
    if (i>=maxbst) { 
    		maxbst= i+16; 
			DebugNTMonitorState [] bsto= bst;
			bst= new DebugNTMonitorState[maxbst];
			java.lang.System.arraycopy(bsto, 0, bst, 0, bsto.length);
			for (int j= bsto.length+1; j<bst.length;j++) {
				bst[j]= null;
			}
		}
    bst[i]= new DebugNTMonitorState(in, out, ct);
  }
  
  public int getBPCount(String nt){
    int i= translator.registNT(nt).id;
    if (i>=maxbst||i<0)
    		return -1;
    return bst[i].bcnt;
  }
  
  public boolean getOnEntry(String nt){
    int i= translator.registNT(nt).id;
    if (i>=maxbst||i<0)
    		return false;
    	return bst[i].in;
  }
  
  public boolean getOnExit(String nt){
    int i= translator.registNT(nt).id;
    if (i>=maxbst||i<0)
    		return false;
    	return bst[i].out;
  }
 

  public void rm(String nt){
    int i= translator.registNT(nt).id;
    bst[i]= null;
  }
  
  public void rmAll(){
    maxbst= 2;
    bst= new DebugNTMonitorState[maxbst];
    bst[0]= null; 
    bst[1]= null;
  }
  
	public void Do(){
	  if (susp) { 
	  		if (par!=null) { 
	  			susp= false; 
	  			synchronized(this) {
	  				notify();
	  			} 
	  			Thread.yield();
	  		}
	  }
	  else { 
			if (par!=null) 
				Thread.yield();
			else { 
				par= new Thread(this); 
				par.start();
			}
    }
  }
              
  public void run(){
    DebugNTProcedureMonitor cnt= new DebugNTProcedureMonitor(this, breakpointFrame);
    Dp4.NTprocMonT.set(getTranslator(), cnt);
    super.run();
    par= null;
//    cstack.removeAllElements();
  }
  
  public void reset(){
	  if (susp) {
	  		if (par!=null) { 
	  		//	par.stop(); 
	  			Thread.currentThread().getThreadGroup().list();
	  		}
	  }
    susp= false;
    par= null;
    Dp4.OP.WrStr(" ## RESET NOT implemented yet ##\n");
  }
  
 /* public void setVisible(boolean b){
  		if (df!=null) 
  			df.setVisible(b);
    super.setVisible(b);
  }*/
  
	public static UsePanel New() {
	  if (count==0) {
			MainData.menuItems[MainData.DEBUG_MENU][MainData.BREAKPOINTS].addActionListener(new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					currentDebugWindow.getDefFrame().setVisible(true);
				}
			});
	  }
	  return new Debug();
	}
}

class DebugNTProcedureMonitor extends NTprocMonT{
  Debug debugWindow;
	DebugBreakpointFrame breakpointFrame;
  int CallLevel= 0, maxCallLevel= 0;
  
  String []    cnts= null; // dummy

	DebugNTProcedureMonitor(Debug d, DebugBreakpointFrame breakpointFrame){
		debugWindow= d;
		this.breakpointFrame= breakpointFrame;
	}

//----------------------- parse -----------------------
  public trgts parse(NonTermT t, Prs p) {
    trgts h;
    int sc= 0;
    Dp4.SourcePos yysrctxt1= null, yysrctxt2= null;
    int i= ((NonTerm)t).id;
    debugWindow.cstack.push(((NonTerm)t).name);
    CallLevel++;
    if (CallLevel>maxCallLevel) maxCallLevel= CallLevel;
    if (debugWindow.bst.length>i&&debugWindow.bst[i]!=null) {
      DebugNTMonitorState bp= debugWindow.bst[i];
      sc= ++bp.cnt;
      p.AC(); 
      yysrctxt1= p.currentSource.GetPos();
      if (sc>=bp.bcnt) { 
			bp.cnt= 0;
				if (bp.in) { 
					Dp4.OP.WrStr("-call "+((NonTerm)t).name+" "+CallLevel+"\n");
					breakpointFrame.show(t, false, p, null);
					breakp();
				}
			}
		}
		h= super.parse(t, p);
		if (debugWindow.bst.length>i&&debugWindow.bst[i]!=null) {
			DebugNTMonitorState bp= debugWindow.bst[i];
			p.AC(); yysrctxt2= p.currentSource.GetPos();
			if (sc>=bp.bcnt) {
				if (bp.out) { 
					Dp4.OP.WrStr("-ret  "+((NonTerm)t).name+" "+CallLevel+" "+p.A+"\n");
					breakpointFrame.show(t, true, p, h);
					breakpointFrame.yysrctxt1= yysrctxt1;
					breakpointFrame.yysrctxt2= yysrctxt2;
					breakp();
				}
			}
		}
		debugWindow.cstack.pop();
		CallLevel--;
		return h;
	}
  
	void breakp(){
		Thread ct= Thread.currentThread();
		if (debugWindow!=null) {
			breakpointFrame.setVisible(true);
			breakpointFrame.toFront();
			debugWindow.ex=new Exception("Breakpoint");
			debugWindow.susp= true;
			if (debugWindow.par!=ct) {
				debugWindow.par=ct;
				Dp4.OP.WrStr("!=\n");
			}
			try {
				synchronized(debugWindow) {
					debugWindow.wait();
				}
			} catch (InterruptedException e){
			}
			breakpointFrame.setVisible(false);
		}
	}
}

class DebugNTMonitorState {
  boolean in= true, out= false;
  int cnt= 0, bcnt= 1;
  
	DebugNTMonitorState(boolean i, boolean o, int c){
		in= i;
		out= o;
		bcnt= c;
	}
}

class DebugBreakpointFrame extends JFrame implements ActionListener{

	protected Debug debugWindow;

  protected NonTermT t= null;
  Prs p= null;
  trgts h= null;
  Dp4.SourcePos yysrctxt1= null, yysrctxt2= null;
  JPanel bts= new JPanel();
  JLabel titelLabel= new JLabel("Stopped", SwingConstants.LEFT);
  JButton B1= new JButton("SourcePos"),
		B2= new JButton("SourceTxt"),
    B3= new JButton("Target"),
		B4= new JButton("NonTerm"),
    B5= new JButton("StackTrace"),
		B6= new JButton("CallStack"),
    GO= new JButton("GO");
    
  public void show(NonTermT t, boolean onExit, Prs p, trgts h) {
  		this.p= p;
  		this.t= t;
  		this.h= h;
  		titelLabel.setText("Stopped at '"+((NonTerm)t).name+"' on "+(onExit?"exit":"entry"));
  		B3.setEnabled(onExit);
  		B2.setEnabled(onExit);
  		if (!onExit)
  			yysrctxt2= null;
  }
    
	public void actionPerformed(ActionEvent evt){
		if (evt.getSource()==B1) {
			p.currentSource.LogPos();
			Dp4.OP.WrStr("\n");
		}else if (evt.getSource()==B2) {
			if (yysrctxt2!=null) {
			  Dp4.OP.WrStr("["
			   +Dp4.Chars.MkSym(Dp4.Chars.appSrc(null, yysrctxt1, yysrctxt2))
			   +"]\n");
		   }
		}else if (evt.getSource()==B3) {
			if (h!=null)
				Dp4.OP.WrStr("["+Dp4.Chars.MkSym(h.trg1)+"]\n");
		}else if (evt.getSource()==B4) { 
			Dp4.OP.WrStr(((NonTerm)t).name
		  	+" from "+((!t.name.equals(t.eqname))?t.eqname+":":"")+((NonTerm)t).impname+"\n");
		}else if (evt.getSource()==B5) {
		  OutputStream b;
		  PrintStream ps= new PrintStream(b= new ByteArrayOutputStream(1000));
		  debugWindow.ex.printStackTrace(ps);
		  Dp4.OP.WrStr(b.toString());
		}else if (evt.getSource()==B6) { 
			Dp4.OP.WrStr("CallStack:\n");
		  for (int i= debugWindow.cstack.size()-1; i>=0; --i) {
		    Dp4.OP.WrStr("  "+i+" "+(String)debugWindow.cstack.elementAt(i)+"\n");
		  }
		}else if (evt.getSource()==GO) { 
			debugWindow.Do();
		} 
	}
	
  DebugBreakpointFrame(Debug debugWindow){
  	super("Inspect");
		this.debugWindow= debugWindow;
    getContentPane().setLayout(new BorderLayout());
    bts.setLayout(new GridLayout(2, 3));
    bts.add(B1);
    bts.add(B2);
    bts.add(B3);
    bts.add(B4);
    bts.add(B5);
    bts.add(B6);
    B1.addActionListener(this);
    B2.addActionListener(this);
    B3.addActionListener(this);
    B4.addActionListener(this);
    B5.addActionListener(this);
    B6.addActionListener(this);
    GO.addActionListener(this);
    getContentPane().add(bts, BorderLayout.CENTER);
    getContentPane().add(GO, BorderLayout.SOUTH);
    getContentPane().add(titelLabel, BorderLayout.NORTH);
    //getContentPane().add(new JLabel(" SourceTxt & Target only on exit "));
    pack();
		Dimension ScS= Toolkit.getDefaultToolkit().getScreenSize();
    //setBounds(0, 60, ScS.width/2, ScS.height/4);
    setLocation(ScS.width/2, ScS.height/4);
  }
}