package Dp4.ide;

/** Depot4 IDE (c) J. Lampe 1999-2003
 *
 * @author Juergen Lampe
 * @version 1.3.3
 * Date: May 29, 2002
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.io.IOException;
import java.net.URL;
import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;


public class HTMLFileViewer extends JFrame {

	private static final long serialVersionUID= -8501275365810952313L;

	/** the editor pane which holds the HTML page */
	private JEditorPane editorPane;


	private boolean dialogForExceptionEnabled;

	/**
	 * Constructs an instance of this class.
	 */
	public HTMLFileViewer(String file) {
		this();
		showFile(file);
	}

	/**
	 * Constructs an instance of this class.
	 */
	public HTMLFileViewer() {
		super("Depot4/Java IDE Help");
		Frame[] frames = Frame.getFrames();
		if (frames.length > 0) {
			for (int i = 0; i < frames.length; i++) {
				if (frames[i].getIconImage() != null) {
					setIconImage(frames[i].getIconImage());
					break;
				}
			}
		}

		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));


		editorPane = new JEditorPane();

		editorPane.setEditable(false);
		editorPane.addHyperlinkListener(new LinkListener());

		JScrollPane scrollPane = new JScrollPane(editorPane);
		scrollPane.setPreferredSize(new Dimension(600, 400));
		mainPanel.add(scrollPane, BorderLayout.CENTER);
		this.getContentPane().add(mainPanel);
		pack();

		dialogForExceptionEnabled = true;
	}

	/**
	 * shows the html content of a given url
	 */
	public void showUrl(URL url) {
		try {
			editorPane.setPage(url);
			setVisible(true);
		}
		catch (IOException x) {
			if (dialogForExceptionEnabled) {
				String msg = "Unable to open help file at " + url + "\n" + x.getMessage();
				JOptionPane.showMessageDialog(this, msg, "ERROR", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	/** Shows an HTML file
	 */
	public void showFile(String fileName) {
		URL fileUrl= getClass().getResource(fileName);
		if (fileUrl!=null){
			showUrl(fileUrl);
		} else {
			String msg = "Unable to find " + fileName + "\n";
			JOptionPane.showMessageDialog(this, msg, "ERROR", JOptionPane.ERROR_MESSAGE);
		}
	}

	/** Enables the dialog for IOExceptions
	 */
	public void setDialogForExceptionEnabled(boolean status) {
		dialogForExceptionEnabled = status;
	}

	public JEditorPane getEditorPane() {
		return editorPane;
	}
	
}

class LinkListener implements HyperlinkListener {

	public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType()==HyperlinkEvent.EventType.ACTIVATED) {
			JEditorPane pane=(JEditorPane)e.getSource();
			if (e instanceof HTMLFrameHyperlinkEvent) {
				HTMLFrameHyperlinkEvent evt=(HTMLFrameHyperlinkEvent)e;
				HTMLDocument doc=(HTMLDocument)pane.getDocument();
				doc.processHTMLFrameHyperlinkEvent(evt);
			} else {
				try {
					pane.setPage(e.getURL());
				} catch (Exception t) {
					String msg = "Unable to open link " + e.getURL() + "\n" + t.getMessage();
					JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(pane), msg, "ERROR", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
}
