/** Depot4 IDE (c) J. Lampe 1999-2010
 *
 *
		* @author	Juergen Lampe
		* @version 1.3.4
		* $Date:$
		* @since JDK 1.2
*/

package Dp4.ide;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import java.io.*;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import Dp4.OP;

public class IdeMain extends JSplitPane implements ChangeListener{

	private static final long serialVersionUID= -4814998310704094496L;

	private int initialSplit;
	private JTabbedPane tp;
	private final static String CONFIG_FILE_NAME = "dp4ide.cfg";
	private final static String VERSION_ID = "#1";
	
	AbstractAction quitAction = new AbstractAction() {
		private static final long serialVersionUID= 499831070401L;

		public void actionPerformed(ActionEvent e) {
			if (MainData.getIt().isStandalone)
				if (checkAllSaved()){
					System.exit(0);
				}
				else
					MainData.getIt().getThisApp().stop();
		}
	};


	public IdeMain() {
		super(JSplitPane.VERTICAL_SPLIT);
		InputMap inputMap = getInputMap(WHEN_IN_FOCUSED_WINDOW);
		Dimension d= Toolkit.getDefaultToolkit().getScreenSize();
		initialSplit= (int)(0.65*d.height);
		//------------ build menu bar ---------------------------------
		// ++++++++++ File +++++++++++++
		JMenu fileMenu= new JMenu("File");
		fileMenu.setMnemonic('F');
		MainData.getIt().menubar.add(fileMenu);
		MainData.menuItems[MainData.FILE_MENU][MainData.NEW]= new JMenuItem("New  Ctrl+N");
		MainData.menuItems[MainData.FILE_MENU][MainData.NEW].setMnemonic('N');
		AbstractAction newAction = new AbstractAction() {
					public void actionPerformed(ActionEvent e) {
						TextDisplay td= MainData.getIt().getActiveTextDisplay();
						if (td!=null)
							td.newFile();
						}
					};
		KeyStroke key = KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK);
		inputMap.put(key, newAction);
		MainData.menuItems[MainData.FILE_MENU][MainData.NEW].addActionListener(newAction);
		fileMenu.add(MainData.menuItems[MainData.FILE_MENU][MainData.NEW]);
		MainData.menuItems[MainData.FILE_MENU][MainData.OPEN]= new JMenuItem("Open Ctrl+O");
		MainData.menuItems[MainData.FILE_MENU][MainData.OPEN].setMnemonic('O');
		MainData.menuItems[MainData.FILE_MENU][MainData.OPEN].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.open();
				}
			});
		fileMenu.add(MainData.menuItems[MainData.FILE_MENU][MainData.OPEN]);
		MainData.menuItems[MainData.FILE_MENU][MainData.SAVE]= new JMenuItem("Save Ctrl+S");
		MainData.menuItems[MainData.FILE_MENU][MainData.SAVE].setMnemonic('S');
		MainData.menuItems[MainData.FILE_MENU][MainData.SAVE].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.save();
				}
			});
		fileMenu.add(MainData.menuItems[MainData.FILE_MENU][MainData.SAVE]);
		MainData.menuItems[MainData.FILE_MENU][MainData.PRINT]= new JMenuItem("Print Ctrl+P");
		MainData.menuItems[MainData.FILE_MENU][MainData.PRINT].setMnemonic('P');
		AbstractAction printAction = new AbstractAction() {
					public void actionPerformed(ActionEvent e) {
							TextDisplay td= MainData.getIt().getActiveTextDisplay();
							if (td!=null)
								print((TextDisplayElement)td);
						}
					};
		MainData.menuItems[MainData.FILE_MENU][MainData.PRINT].addActionListener(printAction);
		key = KeyStroke.getKeyStroke(KeyEvent.VK_P, Event.CTRL_MASK);
		inputMap.put(key, printAction);
		fileMenu.add(MainData.menuItems[MainData.FILE_MENU][MainData.PRINT]);
		MainData.menuItems[MainData.FILE_MENU][MainData.QUIT]= new JMenuItem("Quit  Ctrl+Q");
		MainData.menuItems[MainData.FILE_MENU][MainData.QUIT].setMnemonic('Q');
		MainData.menuItems[MainData.FILE_MENU][MainData.QUIT].addActionListener(quitAction);
		fileMenu.add(MainData.menuItems[MainData.FILE_MENU][MainData.QUIT]);
		key = KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.CTRL_MASK);
		inputMap.put(key, quitAction);
		((JComponent)MainData.getIt().contentPane).getInputMap(WHEN_IN_FOCUSED_WINDOW).put(key, quitAction);
		// ++++++++++ Edit +++++++++++++
		JMenu editMenu= new JMenu("Edit");
		editMenu.setMnemonic('E');
		MainData.getIt().menubar.add(editMenu);
		MainData.menuItems[MainData.EDIT_MENU][MainData.CLEAR]= new JMenuItem("Clear");
		MainData.menuItems[MainData.EDIT_MENU][MainData.CLEAR].setMnemonic('L');
		MainData.menuItems[MainData.EDIT_MENU][MainData.CLEAR].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.clear();
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.CLEAR]);
		MainData.menuItems[MainData.EDIT_MENU][MainData.UNDO]= new JMenuItem("Undo Clear");
		MainData.menuItems[MainData.EDIT_MENU][MainData.UNDO].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.unDo();
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.UNDO]);
		editMenu.add(new JSeparator());
		MainData.menuItems[MainData.EDIT_MENU][MainData.CUT]= new JMenuItem("Cut");
		MainData.menuItems[MainData.EDIT_MENU][MainData.CUT].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.action(TextDisplay.CUT_ACTION, e);
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.CUT]);
		MainData.menuItems[MainData.EDIT_MENU][MainData.COPY]= new JMenuItem("Copy");
		MainData.menuItems[MainData.EDIT_MENU][MainData.COPY].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.action(TextDisplay.COPY_ACTION, e);
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.COPY]);
		MainData.menuItems[MainData.EDIT_MENU][MainData.PASTE]= new JMenuItem("Paste");
		MainData.menuItems[MainData.EDIT_MENU][MainData.PASTE].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.action(TextDisplay.PASTE_ACTION, e);
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.PASTE]);
		editMenu.add(new JSeparator());
		MainData.menuItems[MainData.EDIT_MENU][MainData.GOTO]= new JMenuItem("Goto...");
		MainData.menuItems[MainData.EDIT_MENU][MainData.GOTO].setMnemonic('G');
		MainData.menuItems[MainData.EDIT_MENU][MainData.GOTO].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.gotoPosition();
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.GOTO]);
		editMenu.add(new JSeparator());
		MainData.menuItems[MainData.EDIT_MENU][MainData.FIND]= new JMenuItem("Find...");
		MainData.menuItems[MainData.EDIT_MENU][MainData.FIND].setMnemonic('F');
		MainData.menuItems[MainData.EDIT_MENU][MainData.FIND].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.find();
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.FIND]);
		MainData.menuItems[MainData.EDIT_MENU][MainData.FIND_AGAIN]= new JMenuItem("Find again  F3");
		MainData.menuItems[MainData.EDIT_MENU][MainData.FIND_AGAIN].setMnemonic('G');
		MainData.menuItems[MainData.EDIT_MENU][MainData.FIND_AGAIN].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
		     TextDisplay td= MainData.getIt().getActiveTextDisplay();
		     if (td!=null)
		     	td.findAgain();
				}
			});
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.FIND_AGAIN]);
		editMenu.add(new JSeparator());
		MainData.menuItems[MainData.EDIT_MENU][MainData.FONT]= buildFontMenu();
		editMenu.add(MainData.menuItems[MainData.EDIT_MENU][MainData.FONT]);
		// ++++++++++ Translate +++++++++++++
		JMenu translateMenu= new JMenu("Translate");
		translateMenu.setMnemonic('T');
		MainData.getIt().menubar.add(translateMenu);
		MainData.menuItems[MainData.TRANSLATE_MENU][MainData.ALL]= new JMenuItem("All  Ctrl+T");
		MainData.menuItems[MainData.TRANSLATE_MENU][MainData.ALL].setMnemonic('A');
		MainData.menuItems[MainData.TRANSLATE_MENU][MainData.ALL].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
					if (tp.getSelectedComponent() instanceof AbstractApplicationPanel) {
						AbstractApplicationPanel aw= (AbstractApplicationPanel)tp.getSelectedComponent();
						aw.isSelectionOnly= false;
						aw.Do();
					}
				}
			});
		translateMenu.add(MainData.menuItems[MainData.TRANSLATE_MENU][MainData.ALL]);
		MainData.menuItems[MainData.TRANSLATE_MENU][MainData.SELECTED]= new JMenuItem("Selected");
		MainData.menuItems[MainData.TRANSLATE_MENU][MainData.SELECTED].setMnemonic('L');
		MainData.menuItems[MainData.TRANSLATE_MENU][MainData.SELECTED].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
					if (tp.getSelectedComponent() instanceof AbstractApplicationPanel) {
						AbstractApplicationPanel aw= (AbstractApplicationPanel)tp.getSelectedComponent();
						aw.isSelectionOnly= true;
						aw.Do();
					}
				}
			});
		translateMenu.add(MainData.menuItems[MainData.TRANSLATE_MENU][MainData.SELECTED]);
		translateMenu.add(new JSeparator());
		JMenuItem listItem= new JMenuItem("List all nonterminals");
		listItem.addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
						Dp4.ListNT.list(((AbstractApplicationPanel)tp.getSelectedComponent()).getTranslator());
				}
			});
		translateMenu.add(listItem);

		translateMenu.setEnabled(false);

		// ++++++++++ Options +++++++++++++
		JMenu optionsMenu= new JMenu("Options");
		optionsMenu.setMnemonic('O');
		MainData.getIt().menubar.add(optionsMenu);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.CASE]= new JCheckBoxMenuItem("Case sensitive");
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.CASE].setMnemonic('C');
		optionsMenu.add(MainData.menuItems[MainData.OPTIONS_MENU][MainData.CASE]);

		MainData.menuItems[MainData.OPTIONS_MENU][MainData.CHECK]= new JCheckBoxMenuItem("Check enabled");
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.CHECK].setMnemonic('E');
		optionsMenu.add(MainData.menuItems[MainData.OPTIONS_MENU][MainData.CHECK]);

		MainData.menuItems[MainData.OPTIONS_MENU][MainData.TRACE]= new JCheckBoxMenuItem("Trace");
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.TRACE].setMnemonic('T');
		optionsMenu.add(MainData.menuItems[MainData.OPTIONS_MENU][MainData.TRACE]);

		MainData.menuItems[MainData.OPTIONS_MENU][MainData.WARNINGS]= new JCheckBoxMenuItem("Warnings");
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.WARNINGS].setMnemonic('W');
		optionsMenu.add(MainData.menuItems[MainData.OPTIONS_MENU][MainData.WARNINGS]);

		MainData.menuItems[MainData.OPTIONS_MENU][MainData.VERBOSE]= new JCheckBoxMenuItem("Verbose Compiling");
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.VERBOSE].setMnemonic('B');
		optionsMenu.add(MainData.menuItems[MainData.OPTIONS_MENU][MainData.VERBOSE]);

		MainData.menuItems[MainData.OPTIONS_MENU][MainData.LOG]= new JMenu("Logging");
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.LOG].setMnemonic('L');
		for (int i= 0; i<MainData.logTexts.length; i++) {
			JCheckBoxMenuItem cbitem=  new JCheckBoxMenuItem(MainData.logTexts[i]);
			((JMenu)MainData.menuItems[MainData.OPTIONS_MENU][MainData.LOG]).add(cbitem);
		}
		optionsMenu.add(MainData.menuItems[MainData.OPTIONS_MENU][MainData.LOG]);
		
		optionsMenu.setEnabled(false);
		
		// ++++++++++ Window +++++++++++++
		JMenu windowMenu= new JMenu("Window");
		windowMenu.setMnemonic('W');
		MainData.getIt().menubar.add(windowMenu);
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_USE]= new JMenuItem("Add Use card");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_USE].setMnemonic('U');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_USE].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				AbstractApplicationPanel uw= new UsePanel();
				tp.addTab(uw.id, null, uw, uw.tip);
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_USE]);
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_TRANS]= new JMenuItem("Add Translate card");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_TRANS].setMnemonic('T');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_TRANS].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				AbstractApplicationPanel uw= new TranslatePanel();
				tp.addTab(uw.id, null, uw, uw.tip);
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_TRANS]);
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_FTRANS]= new JMenuItem("Add Trans Files card");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_FTRANS].setMnemonic('f');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_FTRANS].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				AbstractApplicationPanel uw= new TranslateFilesPanel();
				tp.addTab(uw.id, null, uw, uw.tip);
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_FTRANS]);
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_DEBUG]= new JMenuItem("Add Debug card");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_DEBUG].setMnemonic('f');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_DEBUG].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				AbstractApplicationPanel uw= Debug.New();
				tp.addTab(uw.id, null, uw, uw.tip);
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_DEBUG]);
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_ELSE]= new JMenuItem("Add ... card");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_ELSE].setMnemonic('A');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_ELSE].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				addTab();
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.ADD_ELSE]);
		windowMenu.add(new JSeparator());
		MainData.menuItems[MainData.WINDOW_MENU][MainData.DELETE]= new JMenuItem("Delete tab");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.DELETE].setMnemonic('A');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.DELETE].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				deleteTab();
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.DELETE]);
		windowMenu.add(new JSeparator());
		MainData.menuItems[MainData.WINDOW_MENU][MainData.SAVE_CARDS]= new JMenuItem("Save card settings");
		MainData.menuItems[MainData.WINDOW_MENU][MainData.SAVE_CARDS].setMnemonic('V');
		MainData.menuItems[MainData.WINDOW_MENU][MainData.SAVE_CARDS].addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				saveCards();
		}
		});
		windowMenu.add(MainData.menuItems[MainData.WINDOW_MENU][MainData.SAVE_CARDS]);

		windowMenu.setEnabled(true);
		
		// ++++++++++ Debug +++++++++++++
		JMenu debugMenu= new JMenu("Debug");
		debugMenu.setMnemonic('D');
		MainData.getIt().menubar.add(debugMenu);
		MainData.menuItems[MainData.DEBUG_MENU][MainData.BREAKPOINTS]= new JMenuItem("Breakpoints");
		MainData.menuItems[MainData.DEBUG_MENU][MainData.BREAKPOINTS].setMnemonic('B');
		debugMenu.add(MainData.menuItems[MainData.DEBUG_MENU][MainData.BREAKPOINTS]);
		debugMenu.setEnabled(false);
		
		// ++++++++++ Info +++++++++++++
		JMenu infoMenu= new JMenu("Info");
		infoMenu.setMnemonic('I');
		MainData.getIt().menubar.add(infoMenu);
		MainData.menuItems[MainData.INFO_MENU][MainData.HELP]= new JMenuItem("Help IDE");
		MainData.menuItems[MainData.INFO_MENU][MainData.HELP].setMnemonic('H');
		infoMenu.add(MainData.menuItems[MainData.INFO_MENU][MainData.HELP]);
		MainData.menuItems[MainData.INFO_MENU][MainData.HELP_ML4]= new JMenuItem("Help Ml4");
		infoMenu.add(MainData.menuItems[MainData.INFO_MENU][MainData.HELP_ML4]);
		MainData.menuItems[MainData.INFO_MENU][MainData.HELP_EXP]= new JMenuItem("Getting started...");
		infoMenu.add(MainData.menuItems[MainData.INFO_MENU][MainData.HELP_EXP]);
		MainData.menuItems[MainData.INFO_MENU][MainData.HELP_UM]= new JMenuItem("User Manual");
		infoMenu.add(MainData.menuItems[MainData.INFO_MENU][MainData.HELP_UM]);
		infoMenu.add(new JSeparator());
		MainData.menuItems[MainData.INFO_MENU][MainData.ABOUT]= new JMenuItem("About...");
		MainData.menuItems[MainData.INFO_MENU][MainData.ABOUT].setMnemonic('A');
		infoMenu.add(MainData.menuItems[MainData.INFO_MENU][MainData.ABOUT]);

		infoMenu.setEnabled(true);

		tp= new JTabbedPane();
		
			MainData.menuItems[MainData.INFO_MENU][MainData.HELP].addActionListener(new HelpAction(MainData.getIt().getHelpFile()));
			MainData.menuItems[MainData.INFO_MENU][MainData.HELP_ML4].addActionListener(new HelpAction("Ml4Syntax.html"));
			MainData.menuItems[MainData.INFO_MENU][MainData.HELP_EXP].addActionListener(new HelpAction("Example.html"));
			MainData.menuItems[MainData.INFO_MENU][MainData.HELP_UM].addActionListener(new HelpAction("/UM.html"));
		if (MainData.getIt().isStandalone) {
			MainData.menuItems[MainData.INFO_MENU][MainData.ABOUT].addActionListener(new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					AboutWindow.getAboutFrame().show();
			}
			});
		} else {
			tp.addTab("About", null, new Dp4.ide.AboutWindow(), "Show information");
			MainData.menuItems[MainData.INFO_MENU][MainData.ABOUT].addActionListener(new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					 tp.setSelectedIndex(0);
			}
			});
		}

		AbstractApplicationPanel win= new TranslatePanel();
		tp.addTab(win.id, null, win, win.tip);
		win= new TranslateFilesPanel();
		tp.addTab(win.id, null, win, win.tip);
		win= new UsePanel();
		tp.addTab(win.id, null, win, win.tip);
		win= Debug.New();
		tp.addTab(win.id, null, win, win.tip);

		InputStream cfgStream = getClass().getResourceAsStream("/"+CONFIG_FILE_NAME);
		if (cfgStream!=null){
			BufferedReader in= new BufferedReader(new InputStreamReader(cfgStream));
			try {
				String line = in.readLine();
				if (line!=null || line.indexOf(VERSION_ID)==0){
					line = in.readLine();
					int lineNo= 0;
					while (line!=null) {
						String classname;
						int i1= line.indexOf(';');
						if (i1>-1 && lineNo>3){
							classname= line.substring(0, i1);
							addTab(classname);
						}
						int i2= line.indexOf(';', i1+1);
						if (i2>-1){
							AbstractApplicationPanel panel= (AbstractApplicationPanel)tp.getComponentAt(lineNo);
							String fontdsc= line.substring(i1+1, i2);
							int i3= fontdsc.indexOf(',');
							if (i3>-1){
								String fontdsc1= fontdsc.substring(i3+1);
								fontdsc= fontdsc.substring(0, i3);
								panel.setFontDesc(1, fontdsc1);
							}
							panel.setFontDesc(0, fontdsc);
						}
						line = in.readLine();
						lineNo++;
					}
				}
			} catch (IOException e) {

			}

		}


		tp.addChangeListener(this);
		
		setTopComponent(tp);
		TextDisplayElement log= new TextDisplayElement(1, " Log");
		setBottomComponent(log);
		setDividerSize(3);
		
		for (int i= 0; i<tp.getTabCount(); i++) {
			Component component= tp.getComponentAt(i);
			if (component instanceof IdeTab) {  
				((IdeTab)component).setOnTop();
				((IdeTab)component).setBack();
			}
		}

		tp.setSelectedIndex(0);
		stateChanged(new ChangeEvent(this));
		OP.act= new OPTextArea(log.ta);
		MainData.getIt().updateMenu(null);
  }


  public JComponent getMainPanel() {		
		return this;
	}
	
	public void setVisible(boolean b) {
		super.setVisible(b);
		if (b&&initialSplit!=0) {
			setDividerLocation(initialSplit);
			initialSplit= 0;
		}
	}
	
	public void stateChanged(ChangeEvent e) {
		//System.out.println("stateChanged "+e.getSource());
		MainData.getIt().setSelectedTab(tp.getSelectedComponent());
		//	MainData.getIt().switchTextDisplay(null);
	}
	
	void addTab() {
    String name= JOptionPane.showInputDialog("Full name of UsePanel subclass:");
    if (name==null||name.length()<1)
    		return;
		addTab(name);
	}

	private void addTab(String name) {
		AbstractApplicationPanel uw= null;
		try {
			Class ntc= Class.forName(name);
			uw= (AbstractApplicationPanel)ntc.newInstance();
					tp.addTab(uw.id, null, uw, uw.tip);
		}
		catch (Throwable notF) {
			OP.WrStr(Dp4.Messages.systemMessages[0]+name+Dp4.Messages.systemMessages[1]+"\n ---> "
			+notF+"\n");
			return;
		}
	}

	void deleteTab() {
		int idx= tp.getSelectedIndex();
		if (tp.getComponentCount()<=1){
			JOptionPane.showMessageDialog(tp, "You cannot remove the last tab!", "Error", JOptionPane.ERROR_MESSAGE);
		} else if (JOptionPane.showConfirmDialog(tp,
         "Are you sure to delete\n the currently visible tab?",
         	"Confirmation", JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION) {
			if (idx>0)
				tp.setSelectedIndex(idx-1);
			else
				tp.setSelectedIndex(idx+1);
			tp.removeTabAt(idx);
    }
	}

	private void saveCards(){
		File file= new File(CONFIG_FILE_NAME);
		if (file!=null) {
			try {
				PrintStream f= new PrintStream(new FileOutputStream(file));
				if (f!=null) {
					f.println(VERSION_ID);
					int count= tp.getComponentCount();
					for (int j = 0; j<count; j++) {
						AbstractApplicationPanel panel= (AbstractApplicationPanel)tp.getComponentAt(j);
						f.println(panel.getClass().getName()+";"+panel.getFontDesc()+";");
					}
					f.close();
				}
			}	catch (Exception ex) {
				JOptionPane.showMessageDialog(this, "Could not save settings because of\n"+ex,
						"Error", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public boolean checkAllSaved(){
		boolean result= true;
		boolean stillUnsaved= false;
		for (int i= 0; i<tp.getTabCount(); i++) {
			Component component= tp.getComponentAt(i);
			if (component instanceof IdeTab) {
				stillUnsaved= stillUnsaved||((IdeTab)component).isChanged();
			}
		}
		result= !(stillUnsaved && JOptionPane.OK_OPTION!=JOptionPane.showConfirmDialog(tp,
				"There are still unsaved text panes. \nExit anyway?", "Warning", JOptionPane.OK_CANCEL_OPTION,
				JOptionPane.WARNING_MESSAGE));
		return result;
	}

	private static final String[]
		FONT_NAMES= {"Dialog", "DialogInput", "Serif", "SansSerif", "Courier New", "Courier"},
		FONT_STYLES= {"Regular", "Bold", "Italic", "BoldItalic"},
		FONT_SIZES= {"6", "7", "8", "9", "10", "12", "14", "18", "20", "24", "32"};
	private static final int[]		
		STYLE_CODES= {Font.PLAIN, Font.BOLD, Font.ITALIC, Font.BOLD+Font.ITALIC};
		
	private JMenu buildFontMenu() {
		JMenu fontMenu= new JMenu("Font");
		JMenu subMenu= new JMenu("Name");
		fontMenu.add(subMenu);
		JMenuItem item;
		for (int i= 0; i<FONT_NAMES.length; i++) {
			final int fi= i;
			item= new JMenuItem(FONT_NAMES[i]);
			item.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setFontName(FONT_NAMES[fi]);
					}
				}
			);
			subMenu.add(item);
		}
		
		subMenu= new JMenu("Style");
		fontMenu.add(subMenu);
		for (int i= 0; i<FONT_STYLES.length; i++) {
			final int fi= i;
			item= new JMenuItem(FONT_STYLES[i]);
			item.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setFontStyle(STYLE_CODES[fi]);
					}
				}
			);
			subMenu.add(item);
		}
		
		subMenu= new JMenu("Size");
		fontMenu.add(subMenu);
		for (int i= 0; i<FONT_SIZES.length; i++) {
			final int fi= i;
			item= new JMenuItem(FONT_SIZES[i]);
			item.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setFontSize(Integer.parseInt(FONT_SIZES[fi]));
					}
				}
			);
			subMenu.add(item);
		}
		
		return fontMenu;
	}
	
	private void setFontName(String name) {
		TextDisplay td= MainData.getIt().getActiveTextDisplay();
		if (td!=null) {
			Font fold= td.getFont();
			td.setFont(new Font(name, fold.getStyle(), fold.getSize()));
		}
	}
	private void setFontStyle(int style) {
		TextDisplay td= MainData.getIt().getActiveTextDisplay();
		if (td!=null) {
			Font fold= td.getFont();
			td.setFont(fold.deriveFont(style));
		}
	}
	private void setFontSize(int size) {
		TextDisplay td= MainData.getIt().getActiveTextDisplay();
		if (td!=null) {
			Font fold= td.getFont();
			td.setFont(fold.deriveFont((float)size));
		}
	}
	
	public void print(Printable printable) {
		PrinterJob printJob=PrinterJob.getPrinterJob();
		PageFormat pf= printJob.pageDialog(printJob.defaultPage());
		printJob.setPrintable(printable, pf);
		if (printJob.printDialog()) {
			try { 
			  printJob.print();
			} catch (Exception x) {
				x.printStackTrace();
			}
		} else  {
			// canceled
		}
	}

	// ------------------------------------------------------------------------
	/** Help viewer  */
	private HTMLFileViewer helpViewer;
	private class HelpAction extends AbstractAction {
    private final String helpSource;

		public HelpAction(String helpSource) {
			this.helpSource= helpSource;
		}

		public void actionPerformed(ActionEvent e) {
			if (helpViewer==null) {
				helpViewer= new HTMLFileViewer();
				if (MainData.getIt().frame!=null){
					Point location= MainData.getIt().frame.getLocation();
					location.translate(100, 50);
					helpViewer.setLocation(location);
				}
			}
			 helpViewer.showFile(helpSource);
		}
	}

}