/** Depot4 IDE (c) J. Lampe 1999-2010
  *
	* @author	Juergen Lampe
	* @version 1.3.4
	* $Date:$
  */
package Dp4.ide;

import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.util.BitSet;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import Dp4.LogStub;
import Dp4.Translator;
import Dp4.Version;
import Dp4.config;

public class MainData {

  protected static final String Title= " Depot4 for Java "+Version.getVersionString()+" ",
                          IdeVersion=" IDE 1.3.4",
                          VersionDate= "05-MAY-2010",
                          Author= "J. Lampe";

  protected static final int FILE_MENU= 0, EDIT_MENU= 1, TRANSLATE_MENU=2,
  			OPTIONS_MENU= 3, WINDOW_MENU= 4, DEBUG_MENU= 5, INFO_MENU= 6,
  			NEW= 0, OPEN= 1, SAVE= 2, PRINT= 3, QUIT= 4,
  			CLEAR= 0, UNDO= 1, CUT= 2, COPY= 3, PASTE= 4, GOTO= 5, FIND= 6, FIND_AGAIN= 7, FONT= 8,
  			ALL= 0, SELECTED= 1,
  			CASE= 0, CHECK= 1, TRACE= 2, WARNINGS= 3, VERBOSE= 4, LOG= 5,
  			ADD_USE= 0, ADD_TRANS= 1, ADD_FTRANS= 2, ADD_DEBUG= 3, ADD_ELSE= 4, DELETE= 5, SAVE_CARDS= 6,
  			BREAKPOINTS= 0,
				ABOUT= 4, HELP= 0, HELP_ML4= 1, HELP_EXP= 2, HELP_UM= 3;
  protected static JMenuItem[][] menuItems= new JMenuItem[INFO_MENU+1][FONT+1];

  protected static final int[] logBits= {7, 13, 2, 3, 11, 12, /*4,*/ 1, 10};
  protected static final String[] logTexts= {"Timing", "Exception stacks",
  		"Module inst", "Module warnings",
  		"Load exceptions", "Module loading",/* "Globals",*/ "Comment defs", "Lexer settings"};

  protected JApplet curApp;
  protected Container frame;
  protected Container contentPane;
  protected TextDisplay textDisplay;
  protected Component selectedTab;
  protected Vector rootList= new Vector(20);

  public boolean isStandalone= false;
  private Vector childFrames;
  protected JMenuBar menubar;

	protected Translator ml4Translator;

  public JApplet getThisApp() {
      return curApp;
  }

	private static MainData mainData= new MainData();

	public static MainData getIt() {
		return mainData;
	}

  private MainData() {
  		rootList.add("<root>");
  }

	public synchronized Translator getMl4Translator() {
		if (ml4Translator==null)
			ml4Translator= Translator.createTranslator("Ml4-Translator");
		return ml4Translator;
	}

	public String getHelpFile() {
		return "Dp4ideHelp.html";
	}

  public void addAppFrame(JFrame frame) {
       childFrames.addElement(frame);
  }

  public String getParameter(String key, String def) {
    return isStandalone ? System.getProperty(key, def) :
      (curApp.getParameter(key) != null ? curApp.getParameter(key) : def);
  }

  public void init() {
	  LogStub log= LogStub.getLog(getClass().getName());
	  childFrames= new Vector(2, 4);

	  if (isStandalone){
		  config.setConfigurationFileName("Depot4.cfg");
	  } else{
		  config.setImplementation("ident","Dp4.idForApplet");
		  config.setImplementation("id", "Dp4.idForApplet");
	  }
  }

  public void disposeChildFrames() {
    Enumeration en= childFrames.elements();
    while (en.hasMoreElements()) {
      ((Frame)en.nextElement()).dispose();
    }
    childFrames.removeAllElements();
  }

  public void stopChildFrames() {
    Enumeration en= childFrames.elements();
    while (en.hasMoreElements()) {
      ((Frame)en.nextElement()).setVisible(false);
    }
  }

  public void startChildFrames() {
    Enumeration en= childFrames.elements();
    while (en.hasMoreElements()) {
      ((Frame)en.nextElement()).setVisible(true);
    }
  }

  public void setSelectedTab(Component sel) {
  		if (selectedTab!=null&&selectedTab instanceof IdeTab)
  			((IdeTab)selectedTab).setBack();
  		selectedTab= sel;
  		if (selectedTab!=null&&selectedTab instanceof IdeTab)
  			((IdeTab)selectedTab).setOnTop();
  }

  public void switchTextDisplay(TextDisplay to) {
  		if (textDisplay!=null&&textDisplay!=to) {
  			textDisplay.setInactive();
  		}
//  		if (textDisplay!=to)
  		updateMenu(to);

  		textDisplay= to;
  }

  public void updateMenu(TextDisplay td) {
		JMenu fm= menubar.getMenu(FILE_MENU);
		JMenu em= menubar.getMenu(EDIT_MENU);
		if (td!=null){
			fm.getItem(NEW).setEnabled(td.isOpenEnabled());
			fm.getItem(OPEN).setEnabled(td.isOpenEnabled());
			fm.getItem(SAVE).setEnabled(td.isSaveEnabled());
			fm.getItem(PRINT).setEnabled(td.isSaveEnabled());
			em.setEnabled(true);
		} else {
			fm.getItem(NEW).setEnabled(false);
			fm.getItem(OPEN).setEnabled(false);
			fm.getItem(SAVE).setEnabled(false);
			fm.getItem(PRINT).setEnabled(false);
			em.setEnabled(false);
		}
  }

  public TextDisplay getActiveTextDisplay() {
  		return textDisplay;
  }

  protected BitSet storeLogOptions(BitSet result) {
  	result.xor(result);
	  Component[] logs= ((JMenu)menuItems[OPTIONS_MENU][LOG]).getMenuComponents();
	  for (int i= 0; i<logs.length; i++) {
	  		if (logs[i] instanceof AbstractButton) {
	  			if (((AbstractButton)logs[i]).isSelected())
	  				result.set(logBits[i]);
	  		}
	  }
	  return result;
  }

  protected void setLogOptions(BitSet value) {
	  Component[] logs= ((JMenu)menuItems[OPTIONS_MENU][LOG]).getMenuComponents();
	  for (int i= 0; i<logs.length; i++) {
	  		if (logs[i] instanceof AbstractButton) {
	  			((AbstractButton)logs[i]).setSelected(	value.get(logBits[i]))
	  			;
	  		}
	  }
  }

}