/** Depot4 IDE (c) J. Lampe 1999-2003
		TextDisplayElement.java
 *
 * @author  Juergen Lampe
 * @version 1.3.3
 * date:   17.03.2003
 */

package Dp4.ide;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Event;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.InputMap;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import Dp4.OP;
import Dp4.Source;
import Dp4.Streams;
import Dp4.StringStream;
import Dp4.Tools;

public class TextDisplayElement extends JPanel
			implements DocumentListener, FocusListener, TextDisplay, Printable {

	private static final long serialVersionUID= -2090936019415752687L;

	public JTextArea ta= new JTextArea(300, 120);
	protected JScrollPane scrollpane;
	protected JFileChooser chooser;
	private Border activeBorder, passiveBorder;
	private boolean hasOpen= false, hasSave= false;
	private String findString= "";
	private int changeCount= 0;
	private final String defaultTitle;
	private File currentFile;

	final static String newLine= System.getProperty("line.separator");
	static byte[] NLbytes;
	static {
		NLbytes= newLine.getBytes();
	}

	private static Cursor waitCursor= new Cursor(Cursor.WAIT_CURSOR);

	final static Color unchangedActiveBC= new Color(0, 200, 255),
			changedActiveBC= unchangedActiveBC.darker();
	protected Color activeBC= unchangedActiveBC;
	
	public TextDisplayElement(){
		this(0, null);
	}
	public TextDisplayElement(int k){
		this(k, null);
	}
	public TextDisplayElement(String s){
		this(0, s);
	}
	public TextDisplayElement(int k, String s){
        defaultTitle= s;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		/*if (MainData.getIt().isStandalone) */{
			if (k==0||k==2) {
				hasOpen= true;
			}
			hasSave= true;
		}
		ta.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		scrollpane= new JScrollPane(ta);
		add(scrollpane);
		passiveBorder= BorderFactory.createTitledBorder(BorderFactory.createRaisedBevelBorder(), s);
		activeBorder= BorderFactory.createTitledBorder(BorderFactory.createLoweredBevelBorder(), s);
//		activeBorder= BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.black), s);
//		passiveBorder= BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.black), s);
		((TitledBorder)activeBorder).setTitleColor(Color.black);
		((TitledBorder)passiveBorder).setTitleColor(Color.black);
		setBorder(passiveBorder);
		ta.addFocusListener(this);
		ta.getDocument().addDocumentListener(this);
		ta.setFont(new Font("DialogInput", Font.PLAIN, ta.getFont().getSize()));

		InputMap inputMap = ta.getInputMap();
		KeyStroke key = KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				save();
			}
		});	
		key = KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK+Event.SHIFT_MASK);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				currentFile= null;
				save();
			}
		});	
		key = KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				open();
			}
		});	
		key = KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				newFile();
			}
		});	
		key = KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				findAgain();
			}
		});	
		key = KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.CTRL_MASK);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				MainData.menuItems[MainData.FILE_MENU][MainData.QUIT].doClick();
			}
		});	
		key = KeyStroke.getKeyStroke(KeyEvent.VK_T, Event.CTRL_MASK);
		inputMap.put(key, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				MainData.menuItems[MainData.TRANSLATE_MENU][MainData.ALL].doClick();
			}
		});	
	}

	public void setTitle(String title) {
		if (title==null) {
			title= defaultTitle;
		}
		((TitledBorder)activeBorder).setTitle(title);
		((TitledBorder)passiveBorder).setTitle(title);
		repaint();
	}

	private JFileChooser getChooser() {
		String [] txts= {"txt", "Text"};
		if (chooser==null) {
			chooser= new JFileChooser();
			chooser.addChoosableFileFilter(new IdeFileFilter("java", "java source files"));
			chooser.addChoosableFileFilter(new IdeFileFilter(txts, "text files"));
			chooser.addChoosableFileFilter(new IdeFileFilter("ml4", "Ml4 source files"));
			chooser.setCurrentDirectory(new File(System.getProperty("depot4.work.dir", "..")));
		}
		return chooser;
	}
	
	public void open() {
		if (!MainData.getIt().isStandalone){
			JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this), "Action in Applet not allowed");
			return;
		}
		if (changeCount>0) {
			if (JOptionPane.CANCEL_OPTION==JOptionPane.showConfirmDialog(JOptionPane.getFrameForComponent(this), 
               "Text has changed and not been saved.\n You will lose any edits.", "Are you sure?",
               JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE))
				return;
		}
		setTheCursor(waitCursor);
		if(getChooser().showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			currentFile = chooser.getSelectedFile();
			open(currentFile);
			setTitle(chooser.getSelectedFile().getName());
			chooser.setCurrentDirectory(chooser.getSelectedFile());
		}
		setTheCursor(Cursor.getDefaultCursor());
	}
	/**
	 * 
	 */
	protected void setTheCursor(Cursor cursor) {
		if (MainData.getIt().isStandalone) {
			MainData.getIt().frame.setCursor(cursor);
		} else {
			MainData.getIt().getThisApp().setCursor(cursor);
		}
	}
	
	public void save() {
		if (!MainData.getIt().isStandalone){
			JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this), "Action in Applet not allowed");
			return;
		}
		setTheCursor(waitCursor);
		if (currentFile!=null) {
			save(currentFile);
		} else if(getChooser().showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile();
			chooser.getFileFilter().accept(file);
			save(file);
			setTitle(file.getName());
			chooser.setCurrentDirectory(file);
			currentFile= file;
		}
		setTheCursor(Cursor.getDefaultCursor());
	}

	public void newFile(){
		if (!MainData.getIt().isStandalone){
			JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this), "Action in Applet not allowed");
			return;
		}
		clear();
		setTitle(null);
		currentFile= chooser.getSelectedFile();
		if (currentFile!=null){
			String path= chooser.getSelectedFile().getAbsolutePath();
			path= path.substring(0, path.lastIndexOf(File.separatorChar)+1);
			chooser.setCurrentDirectory(null);
			chooser.setCurrentDirectory(new File(path));
		}
	}

	protected String unDoBuffer;
	public void clear() {
		unDoBuffer= ta.getText();
		ta.replaceRange("", 0, ta.getText().length());
		resetChangeCount();
	}
	
	public void unDo() {
		if (unDoBuffer!=null)
			ta.replaceRange(unDoBuffer, 0, ta.getText().length());
	}
	
	public void gotoPosition() {
		int i, j= 0, k= 0;
		String g;//= post.getText();
		g= JOptionPane.showInputDialog(this, "Position:", "Goto", JOptionPane.PLAIN_MESSAGE);
		if (g==null) return; // cancelled
		i= 0;
		k= g.length();
		while(k>i&&!Character.isDigit(g.charAt(i))) i++;
		k= g.indexOf(" ",i); 
		k= (k>=i)?k:g.length();
		if (k-i<1) return;
		j= Integer.parseInt(g.substring(i, k));
		int selectionStart= ta.getSelectionStart();
		int selectionEnd= ta.getSelectionEnd();
		if (selectionEnd-selectionStart>=j) {
			j+= selectionStart;
		}
		if (ta.getText().length()>j-1){
			ta.setCaretPosition(j);
			ta.requestFocus();
		}
	}
	
	public void find() {
		String g= TextInputDialog.showInputDialog(this, "Find what:", "Find", findString);
		if (g==null)
			return; // canceled
		find(g);
		findString= g;
	}
	
	public void findAgain() {
		find(findString);
	}
	
	public void find(String g) {
		int j= -1, k= g.length();
		if (k>0) {
			j= ta.getText().indexOf(g, ta.getCaretPosition());
			if (j>-1){
				ta.setSelectionStart(j);
				ta.setSelectionEnd(j+k);
				ta.requestFocus();
			}
		}
	}
	
	public void open(File arg){
	  int i= 2048;
	  int h, no= 0;
		byte b[]= new byte[1];
		try {
			changeCount= -1;
			FileInputStream f= new FileInputStream(arg);
			h= f.available();
			ta.replaceRange("", 0, ta.getText().length());
			if (h>i) i= h;
			char [] c= new char[i];
			do {
				int j, r= 0;
				for (j=0;j<i&no!=-1;j++) {
				  if ((no= f.read(b))==-1) break;
				  c[j]= (char)b[0];
				  if (c[j]=='\r') { c[j]= '\n'; r++; }
				  else { if (r>0&&c[j]=='\n') j--;
				    r= 0;
				  }
				}
				ta.append(String.copyValueOf(c, 0, j));
			} while (no!=-1);
			f.close();
			ta.select(0, 0);
			ta.setCaretPosition(0);
			ta.requestFocus();
			resetChangeCount();
		}
		catch (Exception ex) {
			Tools.logException(ex, arg.toString());
		}
	}
	
	public void save(File arg){
		if (arg!=null) {
			int i= ta.getText().length();
			byte c []= ta.getText().getBytes();
			try {
				FileOutputStream f= new FileOutputStream(arg);
				if (f!=null) {
					for (int k= 0; k<i; k++) {
						if ((char)c[k]=='\n') f.write(NLbytes);
						else f.write(c[k]);
					}
					f.close();
					resetChangeCount();
				}
			}	catch (Exception ex) {
				Tools.logException(ex, arg.toString());
			}
		}
	}
	
	public Source getSrc() { 
		return StringStream.IString(ta.getText());
	}
	public Source getSrc(boolean sel) {
	  if (sel) {
			String st= ta.getSelectedText();
	    if (st==null||st.length()<=0) OP.WrStr("Nothing selected\n");
	    return Streams.StrmSrc(Dp4.StringStream.OldB(st));
	  } else
			return Streams.StrmSrc(StringStream.OldB(ta.getText()));
	}

	public boolean isSelectionNotEmpty() {
		String st= ta.getSelectedText();
		return st!=null && st.length()>0;
	}

	public void focusGained(FocusEvent e){
		MainData.getIt().switchTextDisplay(this);
		setBackground(activeBC);
		setBorder(activeBorder);
	}
	
	public void focusLost(FocusEvent e){
	}
	
	public boolean isOpenEnabled(){
		return hasOpen;
	}
	
	public boolean isSaveEnabled(){
		return hasSave;
	}
	
	public void setInactive(){
		setBackground(Color.lightGray);
		setBorder(passiveBorder);
	}
	
	public void setFont(Font f) {
		if (ta!=null)
			ta.setFont(f);
	}

	public Font getFont() {
		if (ta!=null)
			return ta.getFont();
		else
			return super.getFont();	
	}
	
	public void action(int type, ActionEvent ev) {
		switch (type) {
			case COPY_ACTION:  ta.copy();
				break;
    		case PASTE_ACTION: ta.paste();
    			break;
    		case CUT_ACTION:   ta.cut();
    			break;
    		default:
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////
	// implement the Printable interface
	///////////////////////////////////////////////////////////////////////////////
	public int print(Graphics g, PageFormat pf, int pi) 
	                                  throws PrinterException {
	
		Graphics2D g2 = (Graphics2D) g;
		g2.setColor(Color.black);
		Font headerFont= new Font("DialogInput", 0, 8);
		g2.setFont(headerFont);
		int headerFontHeight=g2.getFontMetrics().getHeight();
		int headerLineHeight= headerFontHeight;
		int textY= (int)Math.ceil(ta.getMinimumSize().getHeight());
		int pageHeight= (int)pf.getImageableHeight()-2*headerLineHeight;
		int pageWidth= (int)pf.getImageableWidth();
		// paint header
    g2.translate(pf.getImageableX(), pf.getImageableY());
		String page= "Page: "+(pi+1);
    g2.drawString(page,(int)(pageWidth-g2.getFontMetrics().getStringBounds(page, g).getWidth()), headerFontHeight);
		g2.drawLine(0, headerLineHeight+2, pageWidth, headerLineHeight+2);
		// avoid clipping within lines
		g2.setFont(ta.getFont());
		int lineHeight= g2.getFontMetrics().getHeight();
		pageHeight= (pageHeight/lineHeight)*lineHeight;
		int y0= pi*pageHeight+lineHeight/2; // adjust!

	  if (y0 >= textY) {
	     return Printable.NO_SUCH_PAGE;
	  }
		
		ta.getCaret().setVisible(false);
		g2.translate(0f, -y0+2*headerLineHeight);
		g2.setClip(0, y0, pageWidth, pageHeight);
    ta.paint(g2);
		ta.getCaret().setVisible(true);
    return Printable.PAGE_EXISTS;
  }
	
	private void resetChangeCount() {
		changeCount= 0;
		activeBC= unchangedActiveBC;
		if (MainData.getIt().textDisplay==this) {
			setBackground(activeBC);
		}
	}
	
	private void incrementChangeCount() {
		if (changeCount>-1) {
			changeCount++;
			if (changeCount==1) {
				activeBC= changedActiveBC;
				if (MainData.getIt().textDisplay==this) {
					setBackground(activeBC);
				}
			}
		}
	}

	public boolean isChanged() {
		return changeCount>0;
	}

	///////////////////////////////////////////////////////////////////////////////
	// implement the DocumentListener interface
	///////////////////////////////////////////////////////////////////////////////
	public void insertUpdate(DocumentEvent e) {
		incrementChangeCount();
	}
	public void removeUpdate(DocumentEvent e) {
		incrementChangeCount();
	}
	public void changedUpdate(DocumentEvent e) {
	}

}