/** Depot4 IDE (c) J. Lampe 1999-2003
 *
 * A text input dialog.
 * @author Jrgen Lampe
 * @version 1.3.3
 *
 */

package Dp4.ide;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import javax.swing.*;

public class TextInputDialog extends JDialog {
	
	public static final int OK_PRESSED= 0,
		CANCEL_PRESSED= 1,
		CLOSE_PRESSED= 2;
	
	protected static final Cursor waitCursor= new Cursor(Cursor.WAIT_CURSOR);
	
	/** The standard buttons of the dialog. The initial settings can be changed
	 * in <code>constructContent</code>.
	 */
	protected JButton closeButton, okButton, cancelButton;
	
	/** Define which buttons are actually used. The initial settings can be changed
	 * in <code>constructContent</code>.
	 */
	protected boolean hasCloseButton= true, hasCancelButton= true,
		hasOkButton= true;
		
	protected int action;
		
	private JPanel mainP= new JPanel();
	protected JLabel[] lines;
	protected JPanel panel, textPanel;
			
	
	/**
	* Constructs a new AbstractInformationDialog
	*/
	protected TextInputDialog(String title){
		super();
		setTitle(title);
		init();
	}	
		
	/**
	* Constructs a new AbstractInformationDialog
	*/
	protected TextInputDialog(Dialog owner, String title){
		super(owner, title);
		init();
	}	
		
	/**
	* Constructs a new AbstractInformationDialog
	*/
	protected TextInputDialog(Frame owner, String title){
		super(owner, title);
		init();
	}	
	
	/** does the construction */	
	private void init() {
		JPanel bottomP= new JPanel(),
			bottomButtonP= new JPanel(new GridLayout(1, 0, 10, 0));
		mainP.setLayout(new BorderLayout());
		mainP.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
					
		constructContent();
		
		closeButton= new JButton("Close");
		closeButton.setMnemonic('l');
		closeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					TextInputDialog.this.setVisible(false);
					action= CLOSE_PRESSED;
					buttonPressed();
				}
			}
		);
		okButton= new JButton("Ok");
		okButton.setMnemonic('O');
		okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					TextInputDialog.this.setVisible(false);
					action= OK_PRESSED;
					buttonPressed();
				}
			}
		);
		cancelButton= new JButton("Cancel");
		cancelButton.setMnemonic('C');
		cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					TextInputDialog.this.setVisible(false);
					action= CANCEL_PRESSED;
					buttonPressed();
				}
			}
		);
		if (hasOkButton) bottomButtonP.add(okButton);
		if (hasCloseButton) bottomButtonP.add(closeButton);
		if (hasCancelButton) bottomButtonP.add(cancelButton);
		
		bottomP.add(bottomButtonP);
		mainP.add(bottomP, BorderLayout.SOUTH);
		
		getContentPane().add(mainP);
	textfield= new JTextField();
	textfield.addActionListener(textfieldActionListener);

	buildPanel(textfield);
	addWindowListener(new WindowAdapter() {
	  		public void windowActivated(WindowEvent e){
	  			textfield.requestFocus();
	      }        
	  		public void windowOpened(WindowEvent e){
	  			textfield.requestFocus();
	      }        
	 });
	}
	
	/** This method adds a panel to the default dialog.
	 * It should be called in subclass constructors when all its components
	 * are completed.
	 */
	protected final void addContent(JPanel panel) {
		mainP.add(panel, BorderLayout.CENTER);
		panel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createLineBorder(Color.black)
				,
				panel.getBorder()
			)
		);

		pack();
	}
	
	
	/** This method is called in when one of the standard buttons is pressed.
	 * Use the action field to identify which.
	 * Should be overwritten in subclasses.
	 * Current implementation does nothing.
	 */
	protected void buttonPressed() {
	}
	
	/** Utility method. Converts a String with '<code>\\n</code>'s into an array
	 * of Strings.
	 */
	public String[] getArrayOfLines(String src) {
		ArrayList temp= new ArrayList();
		int p= 0, i, l= src.length();
		while (p<l) {
			i= src.indexOf('\n', p);
			if (i==-1)
				i= l;
			temp.add(src.substring(p, i));
			p= i+1;
		}
		String[] result= new String[temp.size()];
		return (String[])temp.toArray(result);
	}
	/** Sets the message text of the dialog. This text may contain \n as line separator.
	 */
	public void setMessage(String text) {
		String[] texts= getArrayOfLines(text);
		textPanel.removeAll();		
		lines= new JLabel[texts.length];
		for (int i= 0; i<texts.length; i++) {
			lines[i]= new JLabel(texts[i]);
			lines[i].setForeground(Color.black);
			textPanel.add(lines[i]);
		}
		pack();
	}
	
	/** Adds an ActionListener to the dialog's ok button.
	 */
	public void addActionListener(ActionListener listener) {
		okButton.addActionListener(listener);
	}
	
	/** Removes an ActionListener from the dialog's ok button.
	 */
	public void removeActionListener(ActionListener listener) {
		okButton.removeActionListener(listener);
	}
	
	/** Constructs the dialog's main panel. */
	protected void buildPanel(JComponent inputElement) {
		JPanel boxpanel= new JPanel();
		boxpanel.setLayout(new BoxLayout(boxpanel, BoxLayout.Y_AXIS));
		boxpanel.add(inputElement);
		boxpanel.add(Box.createVerticalGlue());
		boxpanel.add(Box.createRigidArea(new Dimension(0,45)));
		panel= new JPanel(new BorderLayout());
		panel.add(boxpanel, BorderLayout.CENTER);
		textPanel= new JPanel(new GridLayout(0, 1));
		textPanel.setBorder(BorderFactory.createEmptyBorder(5, 0, 10, 0));
		panel.add(textPanel, BorderLayout.NORTH);
		panel.setBorder(BorderFactory.createEmptyBorder(5, 15, 0, 15));
		addContent(panel);
	}	
	
	/** Is called in the
	 * superclass constructor.
	 */
	protected  void constructContent(){
		hasCloseButton= false;
	}
	protected JTextField textfield;
	
	protected ActionListener textfieldActionListener= new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					 clickOk();
				}
		};
			
	/** Sets the inital text.
	 * @param text  initial text (may be null)
	 */
	public void setText(String text) {
		if (text==null)
			text= "";
		textfield.setText(text);
		textfield.setSelectionStart(0);
		textfield.setSelectionEnd(text.length());
	}
	
	/** Shows the dialog in modal mode.
	 * @param initial  initial text (may be null)
	 * @return  the input text or null (if dialog was cancelled)
	 */
	public String getInput(String initial) {
		setModal(true);
		setText(initial);
		setLocationRelativeTo(getOwner());
		show();
		if (action==OK_PRESSED)
			return textfield.getText();
		else
			return null;
	}
	
	/** Shows a non-modal dialog.
	 * @param initial  initial text (may be null)
	 */
	public void show(String initial) {
		setModal(false);
		setText(initial);
		setLocationRelativeTo(getOwner());
		show();
		textfield.requestFocus();
	}
	
	/** Returns the value of the textfield (for use with nonmodal dialog).
	 * @return  the text
	 */
	public String getText() {
		return textfield.getText();
	}
	
	/** Constructs a modal dialog object, shows it, and returns the input
	 * from the user.
	 * @param parent  the parent frame/dialog (may be null)
	 * @param title  the dialog title
	 * @param text  the dialog's message text
	 * @param value  the initial text
	 * @return  the input text or null (if dialog cancelled)
	 */
	public static String showInputDialog(Component parent, String title, String text,
		String value) {
		TextInputDialog dialog;
		if (parent instanceof Frame)
			dialog= new TextInputDialog((Frame)parent, title);
		else if (parent instanceof Dialog)
			dialog= new TextInputDialog((Dialog)parent, title);
		else
			dialog= new TextInputDialog(title);
		dialog.setMessage(text);
		String result= dialog.getInput(value);
		dialog.dispose();
		return result;
	}
	
	protected void clickOk() {
		 okButton.doClick();
	}
	
}
	