/** Depot4 IDE (c) J. Lampe 1999-2007
    UsePanel.java

    * @author  Juergen Lampe
    * @version 1.3.4
    * @since	 JDK 1.2
    * Date:  8.11.2007
    */
    
package Dp4.ide;

import java.awt.*;
import java.awt.event.*;
import java.util.BitSet;
import javax.swing.*;

import Dp4.Base;
import Dp4.NTprocMonT;
import Dp4.OP;
import Dp4.StrBuf;
import Dp4.Tools;
import Dp4.Translator;
import Dp4.trgts;
import Dp4x.Check;
import Dp4x.Trace;
import Dp4x.indStreamTrg;

public class UsePanel extends AbstractApplicationPanel implements ActionListener, FocusListener {

	private static final long serialVersionUID= 8076333761869007513L;

	private static int count= 0;

	protected JComboBox tf= new JComboBox(MainData.getIt().rootList);
	protected JPanel cbp= new JPanel(),
					subpanf1 = new JPanel(), subpanf = new JPanel();
	protected JSplitPane tap= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

	protected BitSet logOptions= new BitSet(30);
	protected Translator translator;

	public UsePanel() {
		isUseWin= true; 
		tip= "Apply a translator";
		useWin();
		if (!MainData.getIt().isStandalone){
			demo();
			tf.setSelectedItem("nanoPascal2C");
		}
	}
	
	public UsePanel New(JFrame F) {
		return new UsePanel();
	}
	
	void useWin() {                       // ---------- define layout of UsePanel card
		cbp.setLayout(new GridLayout(3,1));
		// assemble panel
		subpanf1.add(new JLabel(" Root: "));
		tf.setPreferredSize(new Dimension(400, tf.getPreferredSize().height));
		tf.setEditable(true);
		tf.addActionListener(this);
		tf.addFocusListener(this);
		tf.setToolTipText("The root production's name; if explicitly placed there, then with full package qualification.");
		subpanf1.add(tf);
		subpanf.setLayout(new FlowLayout(FlowLayout.LEFT));
		subpanf.add(subpanf1);
		// construct window  
		setLayout(new BorderLayout());
		add("North", subpanf);
		src = new TextDisplayElement(0,"  Source: ");
		tap.add(src);
		optionalSecondTextDisplay=new TextDisplayElement(1, "   Target: ");
		tap.add(optionalSecondTextDisplay);
		add("Center", tap);
		tap.setDividerLocation((Toolkit.getDefaultToolkit().getScreenSize().width-40)/2);
	}

	protected String getInstanceName() {
		if (count++!=0)
			return "Use "+count;
		else
			return "Use";
	}

	public void setVisible(boolean b){
		super.setVisible(b);
		if (b) tf.requestFocus();
	}
	
	public void run() {
	  updateData();
	  if (((String)tf.getSelectedItem()).length()==0) { 
	  		OP.WrStr("*** No root specified ***\n"); 
	  		return;
	  }
		getTranslator();
		// ------------------ translate source ------------------------------
		if (isCheck) NTprocMonT.set(translator, new Check()); // interpret check boxes
		if (isTrace) NTprocMonT.set(translator, new Trace());
		NTprocMonT.All(translator);
		for (int i= 0;i<30; i++) 
			if (logOptions.get(i)) 
				Base.LogLevel.set(i);
			else 
				Base.LogLevel.clear(i);       // set protocol level bits
		long t1= System.currentTimeMillis();
		// call translator:
		translator.setCap(isCase);
		trgts tt= Tools.translate(translator,
				src.getSrc(isSelectionOnly),        // construct source object
				((String)tf.getSelectedItem()));    // get root production
		if (translator.isOk()){  // log result
			OP.WrStr(" OK");
		} else {
			OP.WrStr(" ERROR(S)");
		}
		OP.WrLn();
		// write target on text area; new Dp4.FileTrg("file.bla") would do it on file
		if (translator.isOk()) {
			StrBuf strbuf= new StrBuf();       			     		// make StringBuffer stream
//			Dp4.StreamTrg.Tar2Strm(tt.trg1, strbuf);    			// write target on stream - standard without extension
			indStreamTrg.Tar2Strm(tt.trg1, strbuf);             	// use string writer with indentation handling
			optionalSecondTextDisplay.ta.append(strbuf.toString()); // copy string to TextArea
		}
		NTprocMonT.No(translator);  // clear Trace and Check settings
		NTprocMonT.set(translator, null); 
		t1= System.currentTimeMillis()-t1;
		if (logOptions.get(7)){
			OP.WrStr("  "+(t1/60000)+":"+((t1/1000)%60)+"."+((t1%1000)/100)+
				((t1%100)/10)+(t1%10)+" min\n");
		}
		updateMenu();
	}

  public Translator getTranslator() {
		if (translator==null) {
			translator= Translator.createTranslator("Translator of "+getInstanceName());
			IdeTools.addUnloadListener(translator);
			Dp4x.indStreamTrg.install(translator);
		}
		return translator;
  }
 
  public void demo(){
    src.ta.setText("VAR i, j, k;\nBEGIN\n  READ i;\n  j:= 2*i+1;\n  IF j<>i+2 THEN k:=(i+j)DIV 12 ELSE"
    +" k:= 72;\n  WRITE j;\n  WRITE j-2\nEND.");
    tf.setSelectedItem("nanoPascal2C");
  }
  
  public void actionPerformed(ActionEvent e) {
  		if (e.getSource()==tf) {
  			String s= (String)tf.getSelectedItem();
  			if (!MainData.getIt().rootList.contains(s))
  				MainData.getIt().rootList.add(0, s);
  		}
  }
  
  	void updateMenu() {
		super.updateMenu();
		MainData.getIt().menubar.getMenu(MainData.OPTIONS_MENU).setEnabled(true);
		MainData.getIt().menubar.getMenu(MainData.TRANSLATE_MENU).setEnabled(true);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.LOG].setEnabled(true);
		MainData.getIt().setLogOptions(logOptions);
	}
	
  void updateData() {
		super.updateData();
		MainData.getIt().menubar.getMenu(MainData.OPTIONS_MENU).setEnabled(false);
		MainData.getIt().menubar.getMenu(MainData.TRANSLATE_MENU).setEnabled(false);
		MainData.menuItems[MainData.OPTIONS_MENU][MainData.LOG].setEnabled(false);
		MainData.getIt().storeLogOptions(logOptions);
  }

  
  // implement the FocusListener interface (for root combobox)
  public void focusGained(FocusEvent e){
  		tf.getEditor().selectAll();
  }
  public void focusLost(FocusEvent e){
  }

	public boolean isChanged() {
		return src.isChanged()||optionalSecondTextDisplay.isChanged();
	}
}